# $Header:   //Archive/PVCSROOT/Archive_NLS/Mgmt/MVSStatus.ksv   1.13   Jul 24 2001 14:32:36   seirlbec  $

# Now using asirkill to remotely execute a "ps" command locally on the MVSHost.
# This requires that the ASIRemoteExecutionServer service be installed on the
# MVSHost, and that it run as a user that has the privileges necessary to 
# read and write UNC-pathed files.

# MVSStatus.ksh

# Reports the status of all MVS services and queues on a specified host to 
# the standard output.
# Each line is the status of all services and queues for a single MVS image,
# with each field delimited by a pipe. The fields are:
#   Host                          - Name of MVS host
#
#   ListenerPID                   - Last recorded process id of listener
#   LSColor                       - GREEN if CurrentState=DesiredState
#                                   YELLOW if CurrentState=RUNNING when DesiredState<>RUNNING
#                                   otherwise RED if CurrentState<>DesiredState.
#                                   Also if CurrentState and DesiredState are both RUNNING,
#                                   the process id is verified, and the value will be RED if
#                                   the process id is not valid
#   CellSize                      - Number of bytes per entry in queue
#   MaxEntries                    - Maximum number of entries in queue
#   HeadOffset                    - Current head offset
#   TailOffset                    - Current tail offset
#   EnqueueCount                  - Total number of enqueues to queue
#   DequeueCount                  - Total number of dequeues from queue
#   FlushCount                    - Total number of flushes performed on queue
#   Entries                       - Number of entries currently in queue
#   EntriesThresholdAlert         - GREEN if queue <= 30% full
#                                   YELLOW if queue > 30% full and <= 80% full
#                                   RED if queue > 80% full
#                                   RED if queue does not exist
#   EventHandlerSvcStat           - Current status of service
#   EventHandlerDesiredState      - Last recorded DesiredState of service
#   EventHandlerAlert             - GREEN if service status matches desired state,
#                                   YELLOW if service is in a start-pending state with a desired
#                                   state of RUNNING or in a stop-pending state with a desired
#                                   state of STOPPED,
#                                   RED if service status otherwise does not match desired state

# Base path for shell scripts
INSTALL_DIR=$(REG QUERY "SOFTWARE\Accessible Software, Inc.\Access1\1.0\InstallDirectory" | cut -f3)
# Making sure there is a '\' on the end
INSTALL_DIR=$(echo $INSTALL_DIR | awk '/[^\]$/ { $0 = $0 "\\" ; print $0 ; next} // { print $0 }')
BASE_DIR="$INSTALL_DIR"Mgmt\\HMS\\ShellScripts\\

# Base key for Health Monitor Info
PROFILE_BASE_KEY="HKEY_LOCAL_MACHINE\\SOFTWARE\\Accessible Software, Inc.\\Access1\\1.0\\Health Monitor\\Profiles\\"

function Usage {
  print "Usage: MVSStatus [-H<host>] [-v]"
}

verbose=0 # If 1 then verbose messages are generated to standard output during run

if [ -z "$Input" ]; then
  Input=./
fi

# Prints output if verbose option is set, otherwise swallows it.
vprint()
{
  now=$(date +"%Y/%m/%d %H:%M:%S")
  print -R >${Input}LASTACTION.TXT $nNumCycles "| " $now "| " $*
  if [ $verbose = 0 ]; then return; fi
  print -u2 -R >&2 $nNumCycles "| " $now "| " $*
}

while getopts H:K:v c; do
  case $c in
  H)  MVSHost=$OPTARG;;
  K)  HMSHost=$OPTARG;;
  v)  verbose=1;;
  *)  Usage; exit 1;;
  esac
done

# Default MVSHost to local host
if [ -z "$MVSHost" ]; then
  MVSHost=$(hostname)
fi

COMPONENTS_KEY='HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Components'

# Get the ASIMVSEventHandlerSvc status
vprint "Getting ASIMVSEventHandlerSvc status on $MVSHost"
sclist $MVSHost | grep ASIMVSEventHandlerSvc | awk '{
  svcstat=toupper(substr($0, 1, 17));
  sub("[ ]*$", "", svcstat);

  instance=substr($0, 18, 32);
  sub("ASIMVSEventHandlerSvc-", "", instance);
  sub("[ ]*$", "", instance);

  # Lookup DesiredState from registry on MVSHost
  cmd=sprintf("regfind -m \\\\\\\\%s -p \"%s\\ASIMVSEventHandlerSvc\\Instances\\%s\\Hosts\\%s\Status\" -n DesiredState 2>&1 | grep \"DesiredState =\"",
              MVSHost, COMPONENTS_KEY, instance, MVSHost, instance);
  ds = "";
  cmd | getline ds;
  close(cmd);
  sub("[ ]*DesiredState = ", "", ds);
  sub("[ ]*$", "", ds);

  if (ds == "")
    svcstat = "REG KEY NOT FOUND";

  if (ds == "RUNNING")
    if (svcstat == "RUNNING")
      color = "GREEN";
    else if (svcstat == "START PENDING")
      color = "YELLOW";
    else
      color = "RED";
  else if (ds == "STOPPED")
    if (svcstat == "STOPPED")
      color = "YELLOW";
    else if (svcstat == "STOP PENDING")
      color = "YELLOW";
    else
      color = "RED";
  else	
    color = "RED";
  printf("%s|%s|%s\n", instance, svcstat, color);
}' COMPONENTS_KEY="$COMPONENTS_KEY" MVSHost=$MVSHost  > ${MVSHost}_MVSE_Status.txt

# Generate new process list for mvshost
vprint "Generating new process list on $MVSHost"

rm -f //$MVSHost/logs$/ps.log 

#Had to change ps to pstat because ps does not list the full pid number, it chops it
# off at 5 decimals and sometimes they are 6.
asirkill $MVSHost -start "sh -c pstat>//$MVSHost/logs$/ps.log"

# Dump the ASIMVSListenerSvc registry from MVSHost
vprint "Dumping ASIMVSListenerSvc registry from $MVSHost"
regdmp -m '\\'$MVSHost "$COMPONENTS_KEY\ASIMVSListenerSvc" > ${MVSHost}_MVSL_Registry.txt

#Added this because the MVSIPOSListener portion was added in the latest release
grep -i asimvsiposliste //$MVSHost/logs$/ps.log > ${MVSHost}_MVSL_Registry2.txt
regdmp -m '\\'$MVSHost "$COMPONENTS_KEY\ASIMVSIPOSListenerSvc" >> ${MVSHost}_MVSL_Registry2.txt

# Get the ASIMVSListenerSvc status
vprint "Getting ASIMVSListenerSvc status on $MVSHost"
cat ${MVSHost}_MVSL_Registry.txt ${MVSHost}_MVSL_Registry2.txt | awk -v host=$MVSHost ' {
#  printf("%d\n", NR);

  #This if statement takes care of the new MVSIPOSListener, whose functionality
  #is somewhat different from the original SNA version
  if ($1 == "ASIMVSIPOSListe") {
    pid=$2;
    n=getline;
    while (n==1) {
      if ($1 == "Instances") {
        n=getline;
#        printf("  %d\n", NR);
        while (n == 1) {
          if ($0 ~ /^        [A-Z0-9]/) { # beginning of instance
            instance=$1;
            InstanceName[++nInstances] = instance;
            ProcessID[instance] = pid;
            CurrentState[instance] = "RUNNING";
            DesiredState[instance] = "RUNNING";
            n=getline;
#            printf("    %d\n", NR);
            while (n == 1 && $0 !~ /^        [A-Z0-9]/) {
              if ($0 ~ /^    [A-Z0-9]/)
                continue;
              n=getline;
#              printf("      %d\n", NR);
              if ($0 ~ /^    [A-Z0-9]/)
                break;
            }
          }
          if ($0 ~ /^    [A-Z0-9]/)
            break;
        }
      }
      n=getline;
    }
  }  
  if ($1 == "Instances") {
    n=getline;
#    printf("  %d\n", NR);
    while (n == 1) {
      if ($0 ~ /^        [A-Z0-9]/) { # beginning of instance
        instance=$1;
        InstanceName[++nInstances] = instance;
        n=getline;
#        printf("    %d\n", NR);
        while (n == 1 && $0 !~ /^        [A-Z0-9]/) {
          if ($0 ~ /^    [A-Z0-9]/)
            continue;
          if ($1 == "ProcessID") {
            val=toupper($4);
            cmd="echo 16i " val " Aop|dc";
            cmd | getline val;
            ProcessID[instance] = val;
            close(cmd);
          }
          else if ($1 == "CurrentState") {
            #CurrentState[instance] = $3;
            # Since the service status always seems to be
            # stopping, just set it to running
            CurrentState[instance] = "RUNNING";
          }
          else if ($1 == "DesiredState") {
            DesiredState[instance] = $3;
          }
          n=getline;
#          printf("      %d\n", NR);
          if ($0 ~ /^    [A-Z0-9]/)
            break;
        }
      }
      if ($0 ~ /^    [A-Z0-9]/)
        break;
    }
  }
}
END {
  # Get the list of actual listeners running on the host by looking at the process
  # log that was generated before this script was invoked.
  # These show up as "asiserviceapp"
  # cmd2 is for the iposlistener - it only runs one service, but we can use that
  cmd = "cat //" host "/logs$/ps.log | grep -i asiserviceapp"
  cmd2 = "cat //" host "/logs$/ps.log | grep -i asimvsiposliste" 
  
  while (cmd | getline) {
     pids[++npids] = $9;
  }
  while (cmd2 | getline) {
    pids[++npids] = $9;
  }
  
  for (i = 1; i <= nInstances; i++) {
    instance = InstanceName[i];
    pid=ProcessID[instance];
    cs=CurrentState[instance];
    ds=DesiredState[instance];
    if (ds == "RUNNING") {  # Should be running
      if (cs == "RUNNING") { # Claims to be running, so confirm pid
        color="RED";
        for (j = 1; j <= npids; j++) {
          color = "RED"
          if (pids[j] == pid) {
            color = "GREEN";
            break;
          }
        }
      }
      else color="RED";
    }
    else if (cs == "RUNNING") # Claims to be running when it should not be
      color="YELLOW";
    else if (cs == ds && cs != "")
      color="GREEN";
    else
      color="RED";
    printf("%s|%d|%s\n", instance, pid, color);
  }
}' > ${MVSHost}_MVSL_Status.txt

# Get the status of all queues on the MVSHost
vprint "Getting the status of all queues on $MVSHost"

#adding full path name for script...
"$BASE_DIR"fqueuestat //$MVSHost/queues$/*.que | awk -F'|' 'function DisplaySeconds(secs)
{
  hours = int(secs / 3600);
  hfactor = hours*3600;
  minutes = int((secs - hfactor) / 60);
  seconds = secs - hfactor - minutes*60;
  r=sprintf("%02d:%02d:%02d", hours, minutes, seconds);
  return r;
}
{
  name=$5;
  sub("[.][Qq][Uu][Ee]", "", name);

  max_entries=$7;
  entries=$13;
  enqueues=$10;
  dequeues=$11;
  enqueue_age=$17;
  dequeue_age=$18;
  enqueue_age_str=DisplaySeconds(enqueue_age);
  dequeue_age_str=DisplaySeconds(dequeue_age);

  #Inserting if to fix divide by 0 problem, then setting percent_full to 100
  # to indicate error if no max_entries

  # Threshold checks on number of queue entries
  if (max_entries > 0)
    percent_full = (entries / 10) / (max_entries / 1000);
  else
    percent_full = 100;

  if (percent_full > 50)
    ecolor="RED";
  else if (percent_full > 5)
    ecolor="YELLOW";
  else
    ecolor="GREEN";

  # If we have entries, then we check the age of the last dequeue
  if (entries > 0)
    if (dequeue_age > 180)
      dcolor="RED";
    else if (dequeue_age > 60)
      dcolor="YELLOW";
    else
      dcolor = "GREEN";
  else
    dcolor = "GREEN";

  print name "|" max_entries "|" entries "|" ecolor "|" enqueues "|" dequeues "|" enqueue_age_str "|" dequeue_age_str "|" dcolor;
}' > ${MVSHost}_MVSQ_Status.txt

# We only want the queues that belong to the MVS systems, so prepare a list of
# systems from the listener and event handler files
cat ${MVSHost}_MVS[EL]_Status.txt | awk -F'|' '{print $1 "|"}' | sort -u > ${MVSHost}_MVS.lst
fgrep -f ${MVSHost}_MVS.lst ${MVSHost}_MVSQ_Status.txt > foo
mv foo ${MVSHost}_MVSQ_Status.txt

#Here we are adding stuff to monitor those Sender Services
vprint "Getting the ASIMVSSenderSvc status on $MVSHost"

#Dumping out the location of the Sender Services, if it has been set
regdmp -m '\\'$HMSHost "$PROFILE_BASE_KEY$HMSHost\Hosts" | awk '{
  if ($1 == "SenderServiceHost")
    SenderHost = $3
}'

if [ -n $SenderHost ]; then
  SenderHost=$MVSHost
fi

if [ $TECImportHost ] && [ $TECImportHost != "UNUSED" ]; then
    "$BASE_DIR"TECImportStatus -U$TECImportUser -P$TECImportPassword -S$TECImportHost
  fi

sclist $SenderHost | grep ASIMVS | grep SenderSvc | awk '{
  svcstat=toupper(substr($0, 1, 17));
  sub("[ ]*$", "", svcstat);

  instance=substr($0, 18, 32);
  sub("ASIMVSSenderSvc-", "", instance);
  sub("ASIMVSIPSenderSvc-", "", instance);
  sub("[ ]*$", "", instance);

  # Lookup DesiredState from registry on MVSHost
  cmd=sprintf("regfind -m \\\\\\\\%s -p \"%s\\ASIMVSSenderSvc\\Instances\\%s\\Hosts\\%s\Status\" -n DesiredState 2>&1 | grep \"DesiredState =\"",
              SenderHost, COMPONENTS_KEY, instance, SenderHost, instance);
  ds = "";
  cmd | getline ds;
  close(cmd);
  sub("[ ]*DesiredState = ", "", ds);
  sub("[ ]*$", "", ds);

  if (ds == "") {
    cmd=sprintf("regfind -m \\\\\\\\%s -p \"%s\\ASIMVSIPSenderSvc\\Instances\\%s\\Hosts\\%s\Status\" -n DesiredState 2>&1 | grep \"DesiredState =\"",
                SenderHost, COMPONENTS_KEY, instance, SenderHost, instance);
    cmd | getline ds;
    close(cmd);
    sub("[ ]*DesiredState = ", "", ds);
    sub("[ ]*$", "", ds);
    
    if (ds == "")
      svcstat = "REG KEY NOT FOUND";
  }

  if (ds == "RUNNING")
    if (svcstat == "RUNNING")
      color = "GREEN";
    else if (svcstat == "START PENDING")
      color = "YELLOW";
    else
      color = "RED";
  else if (ds == "STOPPED")
    if (svcstat == "STOPPED")
      color = "GREEN";
    else if (svcstat == "STOP PENDING")
      color = "YELLOW";
    else
      color = "RED";
  else	
    color = "RED";
  printf("%s|%s|%s\n", instance, svcstat, color);
}' COMPONENTS_KEY="$COMPONENTS_KEY" SenderHost=$SenderHost  > ${MVSHost}_MVSS_Status.txt


# Output header
print "#Host|Listener|LSColor|Max|Entries|EColor|Enqueues|Dequeues|EnqueueAge|DequeueAge|DColor|EvtHandler|ESColor|SenderSvc|SColor"
print "#-11s |7s |-10s |6s |7s |-7s |9s |9s |11s |11s |-6s |-10s |-7s |-10s |-7s"

vprint "Joining listener, queue, event handler, and sender service status"
# Join the listener, queue, event handler, and sender service status and output to stdout
awk -F'|' '{
  instance=$1;
  if (found_instance[instance] == 0) {
    found_instance[instance] = 1;
    instances[++nInstances] = instance;
  }
  sub("^[^|]*[|]", "", $0);
  if (FILENAME ~ ".*_MVSE_Status.txt")
    MVSE[instance]=$0;
  else if (FILENAME ~ ".*MVSL_Status.txt")
    MVSL[instance]=$0;
  else if (FILENAME ~ ".*MVSQ_Status.txt")
    MVSQ[instance]=$0;
  else if (FILENAME ~ ".*MVSS_Status.txt")
    MVSS[instance]=$0;
}
END {
  for (i = 1; i <= nInstances; i++) {
    instance = instances[i];
    etext = MVSE[instance];
    ltext = MVSL[instance];
    qtext = MVSQ[instance];
    stext = MVSS[instance];
    if (etext == "")
      etext = "NO SVC|RED";
    if (ltext == "")
      ltext = "REG KEY|RED";
    if (qtext == "")
      qtext = sprintf("|NO QUE|RED|||||RED");
    if (stext == "")
      stext = "NO SVC|RED";
    printf("%s|%s|%s|%s|%s\n", instance, ltext, qtext, etext, stext);
  }
}' ${MVSHost}_MVS[ELQS]_Status.txt | sort
#I removed two of the bars from the if (ltext....  statement above
#because they are messing up the output - they are inserted automatically in the printf
#statement


# Remove workfiles
rm -f ${MVSHost}_MVS[ELQS]_Status.txt ${MVSHost}_MVS.lst ${MVSHost}_MVSL_Registry.txt ${MVSHost}_MVSL_Registry2.txt

vprint "MVSStatus finished for $MVSHost"
