# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imDelta.ksv   1.11   Jul 17 2001 15:01:34   lin  $
# $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imDelta.ksv  $
# $Revision:   1.11  $
# $Date:   Jul 17 2001 15:01:34  $
# $Author:   lin  $
#!/bin/sh
#set -x
#
# imDelta.ksh
#   TBSM Install Manager Database & File System objects snapshot collection script
#
#     Each object snapshot will contain the following fields: 
#       host             - hostname of the system where the snapshot was taken
#       context          - either DB version or "FileSystem"  
#       path             - "DB" or the path to a file
#       name             - database object name or the file name
#       objectType       - can be one of the following:
#                           C = CHECK constraint
#                           D = Default or DEFAULT constraint
#                           F = FOREIGN KEY constraint
#                           K = PRIMARY KEY or UNIQUE
#                           P = Sored procedure
#                           S = System table
#                           TR = Trigger
#                           U = User table
#                           V = View
#                           FS = for all files
#       checksum         - sum of the result of sum -r command 
#       blockcount       - block count from the result of sum -r command
#       base_schema_ver  - from sysobjects of the object, or 0 for file system 
#       schema_ver       - from sysobjects of the object, or 0 for file system 
#       crdate           - object creation timestamp
#       mddate           - object last modification timestamp, same as crdate for database objects
#       snapshot version - verion of the snapshot
#       confirmed        - set to 1 the record is added unformed, 0 if otherwise
#       deleted          - set to 0 if object exists, 1 if otherwise
#
#

#
# Lookup the installation directory from the registry
#
INSTALLDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g')

if [ ! -d "$INSTALLDIR" ]; then
  print >&2 "Invalid installation directory: $INSTALLDIR"
  print >&2 "Tivoli Business Systems Manager is not installed on this system"
  exit 1
fi

#
# check for OSQL utility
#
rc=`which osql.exe`; 
rc=$?
if [ $rc -ne 0 ]; then
  echo "ERROR: Could not find ISQL.EXE.  Please install SQL Client Utilities." >&2
  exit 1
fi

#
# Get Database Server, User, Password from Registry
#
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

#
# default values & local variables
#
bTrusted=0
localhost="$(print "$COMPUTERNAME" | awk '{print toupper($0);}')"
currentDate=$(date '+%Y-%m-%d %H:%M:%S.000')
installType="Regular"
installName=$currentDate
batchsize=1000
confirmed_flag=1
invokedFromRemote=0

# temporary files
DBINSERTFILE=$TMPDIR/imDB.insert
SQLCMDFILE=$TMPDIR/imDB.sqlcmd
DBWORKFILE=$TMPDIR/imDB.work
FSWORKFILE=$TMPDIR/imFS.work

cleanUpFiles()
{
  rm -f $DBINSERTFILE
  rm -f $SQLCMDFILE
  rm -f $DBWORKFILE
  rm -f $FSWORKFILE
}

usage   ()
{
  print ''
  print 'Usage: '$0' [options...]'
  print ''
  print '  -S<db_server>           Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>             Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>         Specifies password for database user.  Default=['$db_password'].'
  print '  -E                      Use trusted database connection. Use this instead of -U<username> and -P<password>'
  print ''
  print '  -T                      Specifies the type of the installation. Default to '$installType'.'
  print '  -N                      Specifies the name of the installation. Default to '$installName'.'
  print '  -B                      Specifies the build version of the installation. Default to NULL.'
  print ''
  print ''
}

while getopts "U:S:P:T:N:B:ZEh?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  E)  bTrusted=1;;
  T)  installType=$OPTARG;;
  N)  installName=$OPTARG;;
  B)  buildVersion=$OPTARG;;
  h)  usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

#
# set SQLCMD command
#
if [ "$bTrusted" -eq 1 ]; then
  #SQLCMD="tbsmsql -S$db_server -E -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -E -n -w3000 "
else
  #SQLCMD="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -U$db_user -P$db_password -n -w3000 "
fi


#-------------------------------------------------------------------------------------------------------
#  functions
#-------------------------------------------------------------------------------------------------------

#
# main function to process database objects
#   input: host - hostname
#          path - database name 
processDatabaseObjects()
{
  host=$1
  path=$2

  print "Taking $path database object snapshot from $host."

  #
  # in order to run sp_help or sp_helptext on a remote server
  # we need to set up the remote server as a linked server
  #
  if [ $host != $localhost ]; then
    remote_host_password=$(print $host | awk '{print "sa_"tolower($0)}')
    #RMTSQLCMD="tbsmsql -S$host -Usa -P$remote_host_password -n -w3000 "
    RMTSQLCMD="osql -S$host -Usa -P$remote_host_password -n -w3000 "

    # setup linked server only if it is not done yet
    sql_cmd="SET NOCOUNT ON
      IF NOT EXISTS (SELECT srvname FROM master..sysservers WHERE srvname='$host')
      BEGIN
        PRINT '  ... Creating Linked Server $host'
        EXEC sp_addlinkedserver '$host', 'SQL Server'
        EXEC sp_addlinkedsrvlogin '$host', 'false', NULL, 'sa', '$remote_host_password'
      END
    "
    $SQLCMD -d'Object' -Q"$sql_cmd"  
  else
    # check if localhost name is in the master..sysservers table
    # if not, add it
    sql_cmd="SET NOCOUNT ON
      IF NOT EXISTS (SELECT srvname FROM master..sysservers WHERE srvname='$localhost')
      BEGIN
        PRINT '  ... Creating Server $localhost in the master..sysservers'
        EXEC sp_setnetname '$localhost', '$localhost'
      END
    "
    $SQLCMD -d'Object' -Q"$sql_cmd"  
    RMTSQLCMD=$SQLCMD
  fi

  # prepare database for comparison
  prepareTempSysObjectTable $host $path 
  if [ $? != 0 ]; then 
    return
  fi

  # do database comparison, and generate a work file
  compareDBtable $host $path
  
  # call function to delete them
  delDBObject $host $path $DBWORKFILE

  # call function to add them
  addDBObject $host $path $DBWORKFILE
}


#
#  prepareTempSysObjectTable
#    input: host - hostname
#           path - database name 
#  - Create a temp sysobjects table of the database 
#  - Process exclusion table to remove objects from the temp sysobjects table
#
prepareTempSysObjectTable()
{
  # input parameters
  host=$1
  path=$2

  $SQLCMD -d'Object' -Q"TRUNCATE TABLE imObjectList_current_DB"

  #----------------------------------------------
  # require remote host be a linked server 
  #  FROM Clause has $host.$path.dbo.sysobjects format
  #----------------------------------------------
  
  rm -f $SQLCMDFILE
  print "SET NOCOUNT ON" >> $SQLCMDFILE
  print "SET ANSI_NULLS ON" >> $SQLCMDFILE
  print "SET ANSI_WARNINGS ON" >> $SQLCMDFILE
  print "GO" >> $SQLCMDFILE
  print " 
    INSERT INTO imObjectList_current_DB (host, path, name, objectType, base_schema_ver, schema_ver, crdate)
    SELECT '$host', '$path', name, type, base_schema_ver, schema_ver, crdate
    FROM $host.$path.dbo.sysobjects
    WHERE uid != 3
  " >> $SQLCMDFILE

  $SQLCMD -d'Object' -i$SQLCMDFILE 

  ROWCOUNT=$($SQLCMD -d'Object' -Q"SELECT COUNT(*) FROM imObjectList_current_DB" 2> nul: | awk '/^ *[0-9][0-9]*/ {count=$1} END {print count}')
  
  if [ $ROWCOUNT == 0 ]; then
    print "Cannot access database $path on $host!"
    return 1
  fi

  # do exclusion
  # delete the rows that we don't monitor from the tmp sysobjects table 
  rm -f $SQLCMDFILE
  sql_cmd="SET NOCOUNT ON
    SELECT host, path, name, objectType  
    FROM imExclusionPattern
    WHERE enabled = 1 
      AND contextType = 'DB'
  "
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: |  egrep -v '^[-[:space:]]*$' | while read xhost path name objectType; do
    if [ -z $xhost ]; then
      continue;
    fi

    xhost=$(echo $xhost | awk '{print toupper($0)}')

    print "(host LIKE '"$xhost"' AND path LIKE '"$path"' AND name LIKE '"$name"' AND objectType LIKE '"$objectType"') OR" >> $SQLCMDFILE
  done  # with while loop

  # check if we have anything to exclude
  if [ ! -e $SQLCMDFILE ]; then
    return
  fi

  qstr=$(cat $SQLCMDFILE)
  qstr=$(echo $qstr | awk '/OR$/ {sub("OR$", ""); print $0}')

  sql_cmd="SET NOCOUNT ON
    DELETE FROM imObjectList_current_DB
    WHERE $qstr
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"  
} 
# ---- end of prepareTempSysObjectTable() ----

#
# compareDBtable - compare the temp sysobjects table with the most recent view of imObjectList
#    input: host - hostname
#           path - database name 
#    output: $DBWORKFILE in the following format
#
compareDBtable()
{
  host=$1
  path=$2

  print "
  SET NOCOUNT ON
  -- Get the possible values for the most recent versions of the objects
  SELECT * INTO #imObjectList_v_staging FROM imObjectList
  WHERE (host = '$host' OR host is NULL)
    AND path = '$path'
  
  -- select the highest number version for each object
  SELECT name, objectType, MAX(snapshotver) AS snapshotver 
    INTO #imObjectList_pointer FROM #imObjectList_v_staging
  GROUP BY name, objectType
  
  -- Join the highest number back to the staging table to get all the columns
  SELECT v.* INTO #imObjectList_v
  FROM #imObjectList_v_staging AS v
  INNER JOIN #imObjectList_pointer AS p ON
        v.name        = p.name
    AND v.objectType  = p.objectType
    AND v.snapshotver = p.snapshotver

  -- Look for the record that should be marked deleted
  PRINT '--<DEL START>'

  SELECT v.name, v.objectType, v.checksum, v.blockcount, v.base_schema_ver, v.schema_ver, v.crdate
  FROM imObjectList_current_DB AS c
  RIGHT OUTER JOIN #imObjectList_v AS v ON 
        c.name       = v.name
    AND c.objectType = v.objectType
  WHERE c.name       is NULL
    AND v.deleted    = 0

  PRINT '--<DEL END>'
  
  -- Look for the record to be added or has been changed
  PRINT '--<ADD START>'

  SELECT DISTINCT c.name, c.objectType, 0, 0, c.base_schema_ver, c.schema_ver, c.crdate
  FROM imObjectList_current_DB AS c
  LEFT OUTER JOIN #imObjectList_v AS v ON 
        c.name       = v.name
    AND c.objectType = v.objectType
  WHERE v.name       is NULL
    OR v.deleted     = 1
  
  SELECT c.name, c.objectType, ISNULL(v.checksum, 0), ISNULL(v.blockcount, 0), c.base_schema_ver, c.schema_ver, c.crdate
  FROM imObjectList_current_DB AS c
  INNER JOIN #imObjectList_v AS v ON 
        c.name       = v.name
    AND c.objectType = v.objectType
  WHERE v.deleted    = 0
    AND ((c.base_schema_ver != v.base_schema_ver) OR (c.schema_ver != v.schema_ver) OR (c.crdate != v.crdate))

  PRINT '--<ADD END>'
  " > $SQLCMDFILE.$path
  
  $SQLCMD -d'Object' -i$SQLCMDFILE.$path |  egrep -v '^[-[:space:]]*$' > $DBWORKFILE
}
# ---- end of compareDBtable() ----


#
#  addDBObject() - routine to prepare and add new or changed objects to the imObjectList table.
#    input: host - hostname
#           path - database name 
#           infile - $DBWORKFILE file contains the list of objects to be inserted to the imObjectList 
#
#  - pick up lines in between --<ADD START> and --<ADD END>
#    . generate three lines for each object
#         PRINT '--<START> ...<imObjectList row fields>...'
#         PRINT 'EXEC sp_help <objectname>'   for objectType F, K, S, U
#         or PRINT 'EXEC sp_helptext <objectname>'   for all other objectType 
#         PRINT '--<END>'
#    . append the three lines output to the file $SQLCMDFILE
#
#  - generate the sp_help and sp_helptext by applying $SQLCMDFILE
#  - calculate checksum and blockcount for each sp_help/sp_helptext of the object 
#  - BULK INSERT new records to the imObjectList table 
addDBObject()
{
  # input parameters
  host=$1
  path=$2
  infile=$3

  rm -f $SQLCMDFILE

  # only pick up rows in between --<ADD START> and --<ADD END>
  awk 'BEGIN { onflag=0 } /--<ADD START>/ { onflag=1; next } /--<ADD END>/ { onflag=0; next }
    { if (onflag == 1) print $0 } ' $infile |
  while read name objectType checksum blockcount base_schema_ver schema_ver crdate crtime ; do
    if [ -z $name ]; then
      continue
    fi

    if [ "$objectType" == "F" ] || [ "$objectType" == "K" ] || [ "$objectType" == "S" ] || [ "$objectType" == "U" ]; then 
      print "PRINT '--<START> |$name|$objectType|$checksum|$blockcount|$base_schema_ver|$schema_ver|$crdate $crtime'\
        \nEXEC sp_help $name\nPRINT '--<END> '\nGO" >> $SQLCMDFILE
    else
      print "PRINT '--<START> |$name|$objectType|$checksum|$blockcount|$base_schema_ver|$schema_ver|$crdate $crtime'\
        \nEXEC sp_helptext $name\nPRINT '--<END> '\nGO" >> $SQLCMDFILE
    fi
  
  done

  if [ ! -e $SQLCMDFILE ]; then
    return
  fi

  rm -rf $DBINSERTFILE

  #  $RMTSQLCMD is used here to handle both the local and remote server sp_help or sp_helptext 
  #  output retrieval.  If server is remote, linked server setup is required.
  #  which has been done previously.
  $RMTSQLCMD -d$path -r0 -i$SQLCMDFILE 2> nul: | egrep -v '^[-[:space:]]*$' |
  awk -v snapver=$SNAPVER -v host=$host -v context=$DBCONTEXT -v path=$path -v confirmed_flag=$confirmed_flag -F"|" '
  /--<START> / {
    # --<START> line carry the imObjectList raw fields
    name=$2
    objectType=$3
    checksum=$4
    blockcount=$5
    base_schema_ver=$6
    schema_ver=$7
    crdate=$8
    confirmed=confirmed_flag
    deleted=0
    lineCount=0
    
    next
  }
  /--<END>/ {
    # --<END> indicates the end of sp_help or sp_helptext output
    OFS="|"

    # check if there is any output from sp_help or sp_helptext
    if (lineCount == 0) {
      # set checksum and blockcount to NULL if there is no output line from sp_helptext for that object
      if ((checksum != 0) || (blockcount != 0))
        # record it only when the object has non-null checksum or blockcount previously
        # it is a change from something to nothing
        print host, context, path, name, objectType, "", "", base_schema_ver, schema_ver, crdate, crdate, snapver, confirmed, deleted
    } 
    else {
      # we have output from sp_help or sp_helptext
      close("imDBsum.tmpfile")

      # calculate the current checksum and blockcount, output will be in the format of
      # <sum> <blockcount> <filename> separated by space char
      # getline will put them in $1, $2, $3 
      FS=" "
      "sum -r imDBsum.tmpfile" | getline
      close("sum -r imDBsum.tmpfile")
      FS="|"    

      # we only interested in checksum & blockcount changes
      # record it only when the object has different checksum or blockcount previously
      if (($1 != checksum) || ($2 != blockcount)) {
        print host, context, path, name, objectType, $1, $2, base_schema_ver, schema_ver, crdate, crdate, snapver, confirmed, deleted 
      }
    }
  }
  {
    # this is the output of sp_help or sp_helptext
    # store the line in the imDBsum.tmpfile
    # increment the line count
    print $0 > "imDBsum.tmpfile"
    lineCount += 1
  } 
  ' > $DBINSERTFILE

  # there is no way but hard coded this temp filename
  rm -f imDBsum.tmpfile

  # calculate the record count
  recordCount=0
  if [ -e $DBINSERTFILE ]; then
    recordCount=$(wc -l $DBINSERTFILE | awk '/^ *[0-9][0-9]* / {print $1}')
  fi
 
  # check if we have any work to do
  if [ $recordCount -eq 0 ]; then
    return
  fi

  print "  --Inserting $recordCount new/changed $path object(s) into imObjectList table. "

  # we have the records in final format, bulk insert them into imObjectList table
  sql_cmd="
    SET NOCOUNT ON
    BULK INSERT imObjectList FROM '$DBINSERTFILE'
    WITH (
      BATCHSIZE = $batchsize,
      DATAFILETYPE = 'char',
      FIELDTERMINATOR = '|',
      ROWTERMINATOR = '\n'
    )
  "
  $SQLCMD -d'Object' -Q"$sql_cmd" 
} 
# ---- end of addDBObject() ----


#
#  delDBObject() - routine to add deleted objects to the imObjectList table with deleted field set to 1
#    input: host - hostname
#           path - database name 
#           infile - $DBWORKFILE file contains the list of objects to be mark deleted 
#
#  - pick up lines in between --<DEL START> and --<DEL END>
#  - format lines and set the checksum and blockcount to NULL and the deleted to 0
#  - BULK INSERT new records to the imObjectList table 
delDBObject()
{
  host=$1
  path=$2
  infile=$3
  rm -f $DBINSERTFILE

  # only pick up rows in between --<DEL START> and --<DEL END>
  awk 'BEGIN { onflag=0 } /--<DEL START>/ { onflag=1; next } /--<DEL END>/ { onflag=0; next }
    { if (onflag == 1) print $0 } ' $infile  |
  while read name objectType checksum blockcount base_schema_ver schema_ver crdate crtime ; do
    if [ -z $name ]; then
      continue
    fi
    
    # put them in final format 
    print "$host|$DBCONTEXT|$path|$name|$objectType|||$base_schema_ver|$schema_ver|$crdate $crtime|$crdate $crtime|$SNAPVER|$confirmed_flag|1" >> $DBINSERTFILE
  done

  # calculate the record count 
  recordCount=0
  if [ -e $DBINSERTFILE ]; then
    recordCount=$(wc -l $DBINSERTFILE | awk '/^ *[0-9][0-9]* / {print $1}')
  fi

  # check if there is any record need to be inserted
  if [ $recordCount -eq 0 ]; then
    return
  fi

  print "  --Marking deletion of $recordCount $path database object(s) in imObjectList table. "

  # bulk insert records into database
  sql_cmd="
    SET NOCOUNT ON
    BULK INSERT imObjectList FROM '$DBINSERTFILE'
    WITH (
      BATCHSIZE = $batchsize,
      DATAFILETYPE = 'char',
      FIELDTERMINATOR = '|',
      ROWTERMINATOR = '\n'
    )
  "
  $SQLCMD -d'Object' -Q"$sql_cmd" 
} 
# ---- end of delDBObject() ----


#
#  processFileSystemObjects - routine to process file system objects
#    input: host - hostname
#           fs - starting point of a file system directory 
#  - Use 'ls -1RA' to generate the list of file system objects
#  - Invoke 'FileChk' to populate creation & modification date 
#  - BULK INSERT records of file system object into a temp imObjectList table
#  - Invodke subroutines to continue the process
#
processFileSystemObjects()
{
  host=$1
  fs=$2

  print "Taking $fs File System object snapshot from $host."

  # invoke 'ls; to generate a list of file system object
  # invoke FileChk to retrieve create and last modified date time for each file
  # all directories are excluded from the list
  ls -1RA //$host/$fs | 
  awk -v host=$host ' 
    /^$/ { next } 
    /:$/ { 
      gsub(/:$/, "/") 
      path=$0 
      next 
    } 
    { print path$0 } 
  ' | FileChk | 
  awk -v host=$host ' { 
      nohost="//"host"/"
      newhost=host"|"
      sub(nohost, newhost)
      print $0"|FS" 
    } 
  ' > $DBINSERTFILE.FS

  $SQLCMD -d'Object' -Q"TRUNCATE TABLE imObjectList_current_FS"

  sql_cmd="
    SET NOCOUNT ON
    BULK INSERT imObjectList_current_FS FROM '$DBINSERTFILE.FS'
    WITH (
      BATCHSIZE = $batchsize,
      DATAFILETYPE = 'char',
      FIELDTERMINATOR = '|',
      ROWTERMINATOR = '\n'
    )
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"  

  ROWCOUNT=$($SQLCMD -d'Object' -Q"SELECT COUNT(*) FROM imObjectList_current_FS" 2> nul: | awk '/^ *[0-9][0-9]*/ {count=$1} END {print count}')
  
  if [ $ROWCOUNT == 0 ]; then
    print "Cannot access filesystem $fs on $host!"
    return 1
  fi

  # prepare filesystem table for comparison
  prepareFStable $host $fs 

  # do database comparison, and generate a work file
  compareFStable $host $fs

  # call function to mark deletion 
  delFSObject $host $fs $FSWORKFILE

  # call function to add  new or changed
  addFSObject $host $fs $FSWORKFILE
}


#
#  prepareFStable - prepare the temp file system table
#    input: host - hostname
#           fs - starting point of a file system directory 
#  - Delete the rows that we don't monitor from the tmp table
#
prepareFStable()
{
  # input parameters
  host=$1
  fs=$2

  # do exclusion 
  # delete the rows that we don't monitor from the tmp file system table 

  rm -f $SQLCMDFILE

  sql_cmd="SET NOCOUNT ON
    SELECT host, path, name
    FROM imExclusionPattern
    WHERE enabled = 1 
      AND contextType = 'FS'
  "
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: |  egrep -v '^[-[:space:]]*$' | while read xhost path name ; do
    if [ -z $xhost ]; then
      continue
    fi
    print "(host LIKE '"$xhost"' AND path LIKE '"$path"' AND name LIKE '"$name"') OR" >> $SQLCMDFILE
  done  # with while loop

  # check if we have anything to exclude
  if [ ! -e $SQLCMDFILE ]; then
    return
  fi

  qstr=$(cat $SQLCMDFILE)
  qstr=$(echo $qstr | awk '/OR$/ {sub("OR$", ""); print $0}')

  sql_cmd="SET NOCOUNT ON
    DELETE FROM imObjectList_current_FS 
    WHERE $qstr
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"   
}
# ---- end of prepareFStable() ---


#
#  compareFStable - compare the temp file system table with the most recent view of imObjectList
#    input: host - hostname
#           fs - starting point of a file system directory 
#    output: $FSWORKFILE in the following format
#
#      --<DEL START>
#      <imObjectList raw records to be marked as deleted>
#      --<DEL END>
#      --<ADD START>
#      <imObjectList raw records to be marked as added or changed>
#      --<ADD END>
#
#  - Right Outer Join to generate the deleted list
#  - Left Outer Join to generate the added list
#  - Inner Join to generate the changed list, by check 
#      crdate and mddate
compareFStable()
{
  host=$1
  fs=$2

  print "
  SET NOCOUNT ON
  -- Get the possible values for the most recent versions of the objects
  SELECT * INTO #imObjectList_v_staging FROM imObjectList
  WHERE (host = '$host' OR host is NULL)
    AND path LIKE '$path%'
  
  -- select the highest number version for each object
  SELECT name, path, objectType, MAX(snapshotver) AS snapshotver INTO #imObjectList_pointer FROM #imObjectList_v_staging
  GROUP BY name, path, objectType
  
  -- Join the highest number back to the staging table to get all the columns
  SELECT v.* INTO #imObjectList_v
  FROM #imObjectList_v_staging AS v
  INNER JOIN #imObjectList_pointer AS p ON
        v.name        = p.name
    AND v.path        = p.path
    AND v.objectType  = p.objectType
    AND v.snapshotver = p.snapshotver
  
  -- Look for the record that should be marked deleted
  PRINT '--<DEL START>'

  SELECT v.path + '|' + 
    v.name + '|' + 
    v.objectType + '|' +
    CAST(v.checksum AS NVARCHAR(10)) + '|' +
    CAST(v.blockcount AS NVARCHAR(10)) + '|' +
    CAST(v.base_schema_ver AS NVARCHAR(10)) + '|' +
    CAST(v.schema_ver AS NVARCHAR(10)) + '|' +
    CONVERT(NVARCHAR, v.crdate, 121) + '|' +
    CONVERT(NVARCHAR, v.mddate, 121)
  FROM imObjectList_current_FS AS c
  RIGHT OUTER JOIN #imObjectList_v AS v ON 
        c.name       = v.name
    AND c.path       = v.path
    AND c.objectType = v.objectType
  WHERE c.name       is NULL
    AND v.deleted    = 0

  PRINT '--<DEL END>'

  -- Look for the record to be added or has been changed
  PRINT '--<ADD START>'
  SELECT c.path + '|' + 
    c.name + '|' +
    CONVERT(NVARCHAR, c.crdate, 121) + '|' +
    CONVERT(NVARCHAR, c.mddate, 121)
  FROM imObjectList_current_FS AS c
  LEFT OUTER JOIN #imObjectList_v AS v ON 
        c.name       = v.name
    AND c.path       = v.path
    AND c.objectType = v.objectType
  WHERE v.name       is NULL
    OR v.deleted     = 1

  SELECT c.path + '|' + c.name + '|' +
    CONVERT(NVARCHAR, c.crdate, 121) + '|' +
    CONVERT(NVARCHAR, c.mddate, 121)
  FROM imObjectList_current_FS AS c
  INNER JOIN #imObjectList_v AS v ON 
        c.name       = v.name
    AND c.path       = v.path
    AND c.objectType = v.objectType
  WHERE v.deleted    = 0
    AND (c.crdate != v.crdate OR c.mddate != v.mddate)

    PRINT '--<ADD END>'
  " > $SQLCMDFILE

  $SQLCMD -i$SQLCMDFILE 2> nul: > $FSWORKFILE
}
# ---- end of compareFStable() ---


#
#  addFSObject - routine to add new or changed file system objects to the snapshot table
#    input: host - hostname
#           fs - file system directory name 
#           infile - $FSWORKFILE file contains the list of objects to be added
#  - pick up info of objects from lines in between --<ADD START> and --<ADD END>
#  - calculate the checksum, blockcount of the object
#  - output to $DBINSERTFILE
#  - BULK INSERT to the imObjectList from $DBINSERTFILE
addFSObject()
{
  # input parameters
  host=$1
  fs=$2
  infile=$3

  rm -rf $DBINSERTFILE

  # generate the addition and changed list
  # only pick up rows in between --<ADD START> and --<ADD END>
  awk -F"|" 'BEGIN { OFS="|"; onflag=0 } /--<ADD START>/ { onflag=1; next } /--<ADD END>/ { onflag=0; next } 
    { if (NF == 4 && onflag == 1) { sub("^  *", "", $0); sub("  *$", "", $0); print $0 } } ' $infile |
  awk -v host=$host -v context=$FSCONTEXT -v fs=$fs -v snapver=$SNAPVER -v confirmed_flag=$confirmed_flag -F"|" '
  {
    OFS="|"
    path=$1
    name=$2
    crdate=$3
    mddate=$4

    # calculate checksum & blockcount
    cmd="sum -r \"//"host"/"path"/"name"\""

    FS=" "
    cmd | getline
    close(cmd) 
    $0=$0
    FS="|"

    if (NF == 0)
      print host, context, path, name, "FS", "", "", "", "", crdate, mddate, snapver, confirmed_flag, "0"
    else
      print host, context, path, name, "FS", $1, $2, "", "", crdate, mddate, snapver, confirmed_flag, "0"
  }' > $DBINSERTFILE 2> nul:

  # calculate the record count 
  recordCount=0
  if [ -e $DBINSERTFILE ]; then
    recordCount=$(wc -l $DBINSERTFILE | awk '/^ *[0-9][0-9]* / {print $1}')
  fi 

  # check if there is any record to be processed
  if [ $recordCount -eq 0 ]; then
    return
  fi

  print "  --Inserting $recordCount new/changed $fs object(s) into imObjectList table. "

  # generate the content of snapshot table
  sql_cmd="
    SET NOCOUNT ON
    BULK INSERT imObjectList FROM '$DBINSERTFILE'
    WITH (
      BATCHSIZE = $batchsize,
      DATAFILETYPE = 'char',
      FIELDTERMINATOR = '|',
      ROWTERMINATOR = '\n'
    )
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"
}
# ---- end of addFSObject() ----


#
#  delFSObject - routine to mark deletion of file system objects in the snapshot table
#    input: host - hostname
#           fs - file system directory name 
#           infile - $FSWORKFILE file contains the list of objects to be added
#  - pick up info of objects from lines in between --<DEL START> and --<DEL END>
#  - set the checksum, blockcount to NULL and deleted to 1
#  - output to $DBINSERTFILE
#  - BULK INSERT to the imObjectList from $DBINSERTFILE
#
delFSObject()
{
  host=$1
  fs=$2
  infile=$3
  rm -f $DBINSERTFILE

  # generate the deletion list
  # prepare the file for bulk insert
  # only pick up rows in between --<DEL START> and --<DEL END>
  awk -F"|" '
    BEGIN { OFS="|"; onflag=0 } 
    /--<DEL START>/ { onflag=1; next } 
    /--<DEL END>/ { onflag=0; next } 
    { 
      if (NF == 9 && onflag == 1) { 
        # this is a line with path, name, objectType, checksum, blockcount, base_schema_ver,
        # schema_ver, crdate, mddate
        sub("^  *", "", $0); 
        sub("  *$", "", $0); 
        $4 = ""
        $5 = ""
        print $0 
      } 

  } ' $infile |
  while read dbrecord ; do
    print "$host|$FSCONTEXT|$dbrecord|$SNAPVER|$confirmed_flag|1" >> $DBINSERTFILE
  done

  # calculate record count
  recordCount=0
  if [ -e $DBINSERTFILE ]; then
    recordCount=$(wc -l $DBINSERTFILE | awk '/^ *[0-9][0-9]* / {print $1}')
  fi 

  # check if there is any record to be processed
  if [ $recordCount -eq 0 ]; then
    return
  fi

  print "  --Marking deletion of $recordCount $fs object(s) in imObjectList table. "

  # generate the content of snapshot table
  sql_cmd="
    SET NOCOUNT ON
    BULK INSERT imObjectList FROM '$DBINSERTFILE'
    WITH (
      BATCHSIZE = $batchsize,
      DATAFILETYPE = 'char',
      FIELDTERMINATOR = '|',
      ROWTERMINATOR = '\n'
    )
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"
}
# ---- end of delFSObject() ----


#-------------------------------------------------------------------------------------------------------
# main routine 
#-------------------------------------------------------------------------------------------------------

# validate input of Type, Name and Build
if [ `echo $installType | grep -c "|"` -gt 0 ]; then
  print >&2 "Type cannot contains '|' character"
  exit 3
fi

if [ `echo $installName | grep -c "|"` -gt 0 ]; then
  print >&2 "Name cannot contains '|' character"
  exit 3
fi

if [ `echo $buildVersion | grep -c "|"` -gt 0 ]; then
  print >&2 "Build cannot contains '|' character"
  exit 3
fi

#
# retrieve MAX(snapshotver) and increment it before using it
#   set it to 0 if the table is empty 
#
SNAPVER=$($SQLCMD -d'Object' -Q"SET NOCOUNT ON; SELECT MAX(version) FROM imSnapshotHistory" 2> nul: )

if [ $? != 0 ]; then
  print >&2 "Problem connecting to SQL Server on $db_server!"
  exit 3
fi

SNAPVER=$(echo $SNAPVER | awk '/^ *[0-9][0-9]*/ {ver=$1} END {print ver}')

if [ -z $SNAPVER ]; then
  SNAPVER=0
fi

# increment the version 
SNAPVER=`expr $SNAPVER + 1`

# add a row to the imSnapshotHistory table, set the state field to 'Started'
if [ -z $buildVersion ]; then
  sql_cmd="SET NOCOUNT ON; INSERT INTO imSnapshotHistory (type, name, build, state, date, version) 
    VALUES (N'$installType', N'$installName', NULL, 'Started', '$currentDate', $SNAPVER) 
  "
else
  sql_cmd="SET NOCOUNT ON; INSERT INTO imSnapshotHistory (type, name, build, state, date, version) 
  VALUES (N'$installType', N'$installName', N'$buildVersion', 'Started', '$currentDate', $SNAPVER) 
  "
fi
$SQLCMD -d'Object' -Q"$sql_cmd" 
print "*** Generating object snapshot version $SNAPVER at $currentDate on $localhost."

# set the context field value
DBCONTEXT="$($SQLCMD -Q"SELECT @@VERSION" | awk '/^ Microsoft/ { gsub(/ /, "_"); print $0}')"
FSCONTEXT="FileSystem"

# remove temporary files
cleanUpFiles

# <imObjectSeed> -> prepare database table -> compare new & old -> insert into database 
# step through imObjectSeed table for all path 
sql_cmd="SET NOCOUNT ON 
  SELECT host, contextType, path FROM imObjectSeed 
  WHERE enabled = 1 
  ORDER BY contextType
"
$SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | while read host contextType path; do 
  if [ -z $host ]; then
    continue
  fi

  # convert hostname to upper case
  host=$(echo $host | awk '{print toupper($0)}')

  if [ "$contextType" == "DB" ]; then
    processDatabaseObjects $host $path
  elif [ "$contextType" == "FS" ]; then
    processFileSystemObjects $host $path
  else
    print "Cannot process unknown Context Type $contextType."
    continue
  fi
done 

# add a record with new timestamp and state of complete to the imSnapshotHistory table
now=$(date '+%Y-%m-%d %H:%M:%S.000')
if [ -z $buildVersion ]; then
  sql_cmd="SET NOCOUNT ON; INSERT INTO imSnapshotHistory (type, name, build, state, date, version) 
    VALUES (N'$installType', N'$installName', NULL, 'Completed', '$now', $SNAPVER) 
  "
else
  sql_cmd="SET NOCOUNT ON; INSERT INTO imSnapshotHistory (type, name, build, state, date, version) 
    VALUES (N'$installType', N'$installName', N'$buildVersion', 'Completed', '$now', $SNAPVER) 
  "
fi
$SQLCMD -d'Object' -Q"$sql_cmd" 

exit # end of main
