-- $Header

-- _DiscoverCPSMResourcesTRAN.sql

-- resolve defect #78965
-- Replace join two temp table with join one temp with Transaction_V with added indexes.

USE Object
GO

---------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverCPSMResourcesTRAN' AND type = 'P')
  DROP PROCEDURE _DiscoverCPSMResourcesTRAN
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/_DiscoverCPSMResourcesTRAN.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jul 16 2001 16:23:08  $
-- $Author:   Xiaoyan  $
-- Procedure used by _DiscoverCPSMResources to discover TRAN objects from the CPSMDiscovery table
CREATE PROCEDURE _DiscoverCPSMResourcesTRAN
  @cid          ClassID,
  @id           ObjID,
  @batchid      int,
  @incremental  TINYINT = 0,
  @verbose      TINYINT = 0
AS
  -- If there is no outer transaction then begin a transaction, otherwise create
  -- a transaction savepoint
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _DiscoverCPSMResourcesTRAN
  ELSE
    SAVE TRANSACTION _DiscoverCPSMResourcesTRAN

  -- Perform garbage-collection on tables we reference here so that we don't
  -- have to pay attention to deleted flags

  DELETE Transaction_C WHERE deleted = 1
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesTRAN
    RETURN -30400
  END

  -- temp table for holding discovered TRAN records
  CREATE TABLE #TRAN
  (
    CICSName      ObjName,
    cid           NVARCHAR(4),
    TranClass     NVARCHAR(40) NULL,
    TranId        NVARCHAR(40) NULL,
    REMOTENAME    NVARCHAR(40) NULL,
    REMOTESYSTEM  NVARCHAR(40) NULL,
    STATE1        NVARCHAR(40) NULL,
    cics_id       ObjID,
    tran_id       ObjID NULL
  )
  INSERT INTO #TRAN
    SELECT
      dtran.CICSName,
      dtran.cid,
      dtran.TranClass,
      dtran.TranId,
      dtran.REMOTENAME,
      dtran.REMOTESYSTEM,
      CONVERT(CHAR(3),ISNULL(dtran.STATE1,'')),
      dcics.obj_id,
      NULL
    FROM
      CPSMDiscovery AS dtran
      JOIN CPSMDiscovery AS dcics ON
        dtran.batchid = dcics.batchid
        AND ((dtran.CICSPLEX IS NULL AND dcics.CICSPLEX IS NULL) OR dtran.CICSPLEX = dcics.CICSPLEX)
        AND ((dtran.SystemId IS NULL AND dcics.SystemId IS NULL) OR dtran.SystemId = dcics.SystemId)
--        AND ((dtran.MVSID IS NULL AND dcics.MVSID IS NULL) OR dtran.MVSID = dcics.MVSID)
        AND dtran.CICSName = dcics.CICSName
        AND dcics.obj_id IS NOT NULL
    WHERE
      dtran.batchid = @batchid
      AND dtran.cid IN ('CITL', 'CITR') -- local and remote transactions
      AND dcics.cid = 'CIMA'
  
  -- Update transaction ids
  UPDATE t SET tran_id = prev.id
  FROM
    #TRAN AS t
      JOIN Transaction_V AS prev ON prev._ParentCID = 'CICS' AND prev._ParentID = t.cics_id AND prev.name = t.TranId

  -- Update the attributes of any previously existing transactions
  IF @@ROWCOUNT > 0
  BEGIN
    UPDATE prev SET
      _TName = t.TranId,
      _TClass = t.TranClass,
      _CurrentStateID = ISNULL(sttp.id, 0)
    FROM
      Transaction_V AS prev
        JOIN #TRAN AS t ON t.tran_id = prev.id
        LEFT OUTER JOIN StateType_V AS sttp ON sttp.name LIKE 'TRAN%' AND UPPER(sttp._State) = UPPER(t.STATE1)
    WHERE
      prev._TName <> t.TranId
      OR prev._TClass <> t.TranClass
      OR prev._CurrentStateID <> sttp.id
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesTRAN
      RETURN -30402
    END
  END

  -- Any records in #TRAN with null tran_ids indicate objects that need to be created
  DECLARE @nTrans INT
  SELECT @nTrans = COUNT(*) FROM #TRAN WHERE tran_id IS NULL
  IF @nTrans > 0
  BEGIN
    IF @verbose = 1
      PRINT 'Creating ' + CAST(@nTrans AS NVARCHAR(9)) + ' transactions'

    DECLARE @base_id ObjID
    EXEC @base_id = alloc_TRAN @nTrans
    IF @base_id < 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesTRAN
      RETURN -30403
    END

    CREATE TABLE #NEW_TRAN
    (
      cics_id   ObjID,
      TranId    NVARCHAR(40),
      rowno     INT IDENTITY(0,1)
    )
    INSERT #NEW_TRAN(cics_id, TranId) SELECT cics_id, TranId FROM #TRAN WHERE tran_id IS NULL

    IF @verbose = 1
      SELECT @base_id+n.rowno AS id, t.TranId AS name, 'CICS' AS _ParentCID, t.cics_id AS _ParentID,
        ISNULL(sttp.id,0) AS _CurrentStateID,
        t.TranId AS _TName,
        t.TranClass AS _TClass
      FROM #NEW_TRAN AS n JOIN #TRAN AS t ON t.cics_id = n.cics_id AND t.TranId = n.TranId
        LEFT OUTER JOIN StateType_V AS sttp ON sttp.name LIKE 'TRAN%' AND UPPER(sttp._State) = UPPER(t.STATE1)

    INSERT Transaction_C(id, name, _ParentCID, _ParentID, _CurrentStateID, _TName, _TClass)
      SELECT @base_id+n.rowno, t.TranId, 'CICS', t.cics_id, ISNULL(sttp.id,0), t.TranId, t.TranClass
      FROM #NEW_TRAN AS n JOIN #TRAN AS t ON t.cics_id = n.cics_id AND t.TranId = n.TranId
        LEFT OUTER JOIN StateType_V AS sttp ON sttp.name LIKE 'TRAN%' AND UPPER(sttp._State) = UPPER(t.STATE1)
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION _DiscoverCPSMResourcesTRAN
      RETURN -30404
    END

    -- Update transaction ids in temp table
    UPDATE t SET tran_id = c.id 
      FROM #TRAN AS t
      JOIN Transaction_C AS c ON c._ParentCID = 'CICS' 
      AND c._ParentID = t.cics_id 
      AND c.name = t.TranId
  END

  IF @verbose = 1
    SELECT '#TRAN' AS '#TRAN', * FROM #TRAN

  -- Update the CPSMDiscovery table with the ids of the discovered objects
  UPDATE dtran SET
    obj_cid = 'TRAN',
    obj_id = t.tran_id
  FROM
    CPSMDiscovery AS dtran
      JOIN CPSMDiscovery AS dcics ON
        dcics.batchid = dtran.batchid
        AND dcics.cid = 'CIMA'
      JOIN #TRAN AS t ON
        dcics.obj_id = t.cics_id
        AND t.TranId = dtran.TranId
  WHERE
    dtran.batchid = @batchid AND dtran.cid IN ('CITL', 'CITR')
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION _DiscoverCPSMResourcesTRAN
    RETURN -30405
  END

  IF @verbose = 1
    SELECT '#TRAN' AS '#TRAN', * FROM #TRAN

  -- If we're the outer transaction then commit
  IF @tcount = 0
    COMMIT TRANSACTION _DiscoverCPSMResourcesTRAN

GO
