SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_createLinks]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_createLinks]
GO




/* This procedure handles creating links from the data in a temporary table. First the table is filtered to      
   eliminate unsupported link types, then each row is processed and a link created if one doesn't already exist.  
   The CL_LinkRulesToLinks table is updated to reflect any new links that are created.

   The temporary table is defined as follows:                                                                    

     CREATE TABLE #linkCandidates( name               NVARCHAR(128),
                                   src_direction      TINYINT,       
                                   src_cid            ClassID,
                                   src_id             ObjID,
                                   dst_direction      TINYINT,       
                                   dst_cid            ClassID,
                                   dst_id             ObjID,
                                   rule_id            INT )
 */

CREATE PROCEDURE cl_createLinks            
AS

RAISERROR( 'cl_createLinks', 0, 1 )                                               

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_createLinks                        
ELSE
  SAVE TRAN cl_createLinks                        


DECLARE @name               NVARCHAR(128) 
DECLARE @src_direction      TINYINT
DECLARE @src_cid            ClassID 
DECLARE @src_id             ObjID 
DECLARE @dst_direction      TINYINT
DECLARE @dst_cid            ClassID 
DECLARE @dst_id             ClassID 

DECLARE @rule_id            INT
DECLARE @link_id            INT

DECLARE @needRollback       BIT


-- eliminate entries where the destination CID does not support the relationship and direction
DELETE FROM #linkCandidates WHERE EXISTS ( SELECT * FROM CL_CIDLinks  WHERE cid = dst_cid ) AND
                            NOT EXISTS ( SELECT * FROM CL_CIDLinks WHERE cid = dst_cid  AND 
                                         CL_CIDLinks.name = #linkCandidates.name        AND 
                                         CL_CIDLinks.direction = #linkCandidates.dst_direction )
IF @@ERROR <> 0 BEGIN
  RAISERROR( 'error deleting from temp table #linkCandidates', 0, 1 )
  ROLLBACK TRAN cl_createLinks
  RETURN -30000
END


SET @needRollback = 0

-- cursor through the input table                                 
DECLARE linkCursor CURSOR FOR SELECT * FROM #linkCandidates 

OPEN linkCursor
FETCH NEXT FROM linkCursor INTO @name,
                                @src_direction,           
                                @src_cid,      
                                @src_id,      
                                @dst_direction,           
                                @dst_cid,     
                                @dst_id,  
                                @rule_id

WHILE @@FETCH_STATUS = 0 AND @needRollback = 0 BEGIN

  -- search for an existing link. This requires checking for an existing link that matches in terms of direction,
  -- source, and destination, or that matches in "reverse".
  SELECT @link_id = id FROM LinkObject_V obj JOIN containment c ON
                            c.dst_cid = 'LNKX' AND c.dst_id = obj.id WHERE
                            c.deleted = 0 AND  @name = obj.name             AND
                                               ( ( @src_direction = obj._src_direction  AND
                                                   @src_cid = obj._src_cid              AND
                                                   @src_id = obj._src_id                AND
                                                   @dst_cid = obj._dst_cid              AND
                                                   @dst_id = obj._dst_id )  OR   
                                                 ( @src_direction = obj._dst_direction  AND
                                                   @src_cid = obj._dst_cid              AND
                                                   @src_id = obj._dst_id                AND
                                                   @dst_cid = obj._src_cid              AND
                                                   @dst_id = obj._src_id ) )     

  -- if there is not an existing link, create one
  IF @link_id IS NULL BEGIN
    
    EXEC @link_id = asisp_createLNKX @name = @name,
                                     @parent_cid = 'STCR',
                                     @parent_id = 1,
                                     @src_direction = @src_direction,
                                     @src_cid = @src_cid,
                                     @src_id = @src_id,
                                     @dst_direction = @dst_direction,
                                     @dst_cid = @dst_cid,
                                     @dst_id = @dst_id,
                                     @isTopo = 1

    IF @link_id > 0           
      RAISERROR( 'a link object has been added with link id=%d', 0, 1, @link_id )

  END

  -- for a valid link id, add an entry pairing the rule id with the link id
  IF @link_id > 0 BEGIN

    IF NOT EXISTS ( SELECT * FROM CL_LinkRulesToLinks WHERE link_id = @link_id AND rule_id = @rule_id ) BEGIN

      INSERT INTO CL_LinkRulesToLinks VALUES( @rule_id, @link_id )

      IF @@ROWCOUNT = 1 
        RAISERROR( 'a CL_LinkRulesToLinks entry has been added for rule id=%d and link id=%d',
                   0, 1,
                   @rule_id, @link_id )
      ELSE BEGIN
        RAISERROR( 'error trying to add CL_LinkRulesToLinks entry for rule id=%d and link id=%d',
                   0, 1,
                   @rule_id, @link_id )
        SET @needRollback = 1
      END

    END

  END
  ELSE BEGIN
    RAISERROR( 'error creating Link Object', 0, 1 )
    SET @needRollback = 1
  END

  FETCH NEXT FROM linkCursor INTO @name,
                                  @src_direction,           
                                  @src_cid,      
                                  @src_id,      
                                  @dst_direction,           
                                  @dst_cid,     
                                  @dst_id,  
                                  @rule_id

END
CLOSE linkCursor
DEALLOCATE linkCursor


IF @needRollback = 1 BEGIN
  ROLLBACK TRAN cl_createLinks
  RETURN -30000
END

-- commit transaction
IF @tcount = 0                           
  COMMIT TRAN cl_createLinks

RETURN 0



GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

