SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_deleteLinksContaining]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_deleteLinksContaining]
GO




/* This procedure handles deleting all links that contain the specified resource. The deleting happens in two stages:

   1) All link rules with a matching src_cid and src_id are retrieved. For each rule, a procedure is called that
      deletes the links associated with a certain rule. Then a procedure is called to delete the rule itself.

   2) All link ids are retrieved for which the link's destination equals the specified resource. For each link id,
      the LNKX object class instance is deleted, then all entries in CL_LinkRulesToLinks with a matching link id are
      deleted.
 */

CREATE PROCEDURE cl_deleteLinksContaining @cid         ClassID,
                                          @id          ObjID
AS

RAISERROR( 'cl_deleteLinksContaining: @cid=%s, @id=%d', 0, 1, @cid, @id )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_deleteLinksContaining              
ELSE
  SAVE TRAN cl_deleteLinksContaining              


DECLARE @rule_id            INT
DECLARE @link_id            INT
DECLARE @return             INT

-- locate all the rules having the specified resource as the source
DECLARE linkRuleCursor CURSOR FOR 
                       SELECT rule_id FROM CL_LinkRules WHERE src_cid = @cid AND src_id = @id

OPEN linkRuleCursor
FETCH NEXT FROM linkRuleCursor INTO @rule_id

SET @return = 0

WHILE @@FETCH_STATUS = 0 AND @return !< 0 BEGIN

  -- delete the links by rule id, then delete the link rule
  EXEC @return = cl_deleteLinksForRule @rule_id
  IF @return !< 0 BEGIN
    EXEC @return = cl_deleteLinkRule @rule_id
    FETCH NEXT FROM linkRuleCursor INTO @rule_id
  END

END
CLOSE linkRuleCursor
DEALLOCATE linkRuleCursor


IF @return !< 0 BEGIN        -- any need to continue?
  -- locate all the links having the specified resource as the destination
  DECLARE linkObjectCursor CURSOR FOR 
                           SELECT id FROM LinkObject_V obj JOIN containment c ON
                           c.dst_cid = 'LNKX' AND c.dst_id = obj.id WHERE
                           c.deleted = 0 AND @cid = obj._dst_cid AND @id = obj._dst_id        


  OPEN linkObjectCursor
  FETCH NEXT FROM linkObjectCursor INTO @link_id

  WHILE @@FETCH_STATUS = 0 AND @return !< 0 BEGIN

    EXEC @return = delete_LNKX @link_id

    IF @return < 0 BEGIN
      RAISERROR( 'error returned from delete_LNKX for link id=%d', 0, 1, @link_id )
    END
    ELSE BEGIN
      RAISERROR( 'deleted link object (LNKX) with instance id=%d', 0, 1, @link_id )

      -- delete the "link rules to links" entries
      DELETE FROM CL_LinkRulesToLinks WHERE link_id = @link_id

      IF @@ERROR <> 0 BEGIN
        RAISERROR( 'error deleting from CL_LinkRulesToLinks for link id=%d', 0, 1, @link_id )
        SET @return = -30000 
      END
      ELSE BEGIN
        RAISERROR( 'deleted from CL_LinkRulesToLinks for link id=%d', 0, 1, @link_id )
        FETCH NEXT FROM linkObjectCursor INTO @link_id
      END

    END


  END
  CLOSE linkObjectCursor
  DEALLOCATE linkObjectCursor
END

IF @return < 0 
  ROLLBACK TRAN cl_deleteLinksContaining
ELSE 
  -- commit transaction
  IF @tcount = 0                           
    COMMIT TRAN cl_deleteLinksContaining

RETURN @return




GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

