PRINT 'Creating procedure cl_processBulkLinks'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processBulkLinks')
  DROP PROCEDURE cl_processBulkLinks
GO

CREATE PROCEDURE cl_processBulkLinks
@trid INT,
@instrid NVARCHAR(255)
AS
  -- All the attributes are in #events table
  RAISERROR('cl_processBulkLinks: @trid=%d, @instrid=%s', 0, 1, @trid, @instrid)

  -- table for parsing and getting all link combinations
  CREATE TABLE #contd_links (
    src_id ObjID,
    dst_cid ClassID,
    dst_kn NVARCHAR(64),
    dst_kv NVARCHAR(255)
  )

  -- input table for link rules
  CREATE TABLE #linkEventData ( 
    cid 		ClassID,
    iid	 		ObjID,
    dest_cid 		ClassID NULL,
    linkname 		NVARCHAR(128),
    src_direction 	TINYINT,  
    dest_direction 	TINYINT,
    dest_keyname 	NVARCHAR(32),
    dest_condition 	NVARCHAR(512),
    enabled 		BIT,
    excl 		BIT,
    priority 		INT,
    action 		INT
  )

  -- input table to process containment links
  CREATE TABLE #linkCandidates ( 
    name               NVARCHAR(128),
    src_direction      TINYINT,    
    src_cid            ClassID,
    src_id             ObjID,
    dst_direction      TINYINT,    
    dst_cid            ClassID,
    dst_id             ObjID,
    rule_id            INT 
  )

  -- table of dst_oids if they exist
  CREATE TABLE #dst_oids (
    oid ObjID
  )
  
  DECLARE @ret INT, @name NVARCHAR(255), @dir NVARCHAR(255), @src_cid ClassID, @cok NVARCHAR(32)
  DECLARE @dst_cid ClassID, @kn NVARCHAR(64), @kv NVARCHAR(255), @evtid INT, @cstr NVARCHAR(32)
  DECLARE @disp VARCHAR(8000), @stat NVARCHAR(8)
  SET @cstr = 0
  SET @cok = 0

  -- Convert link direction from string to integer
  -- tosrc(1), todst(2), 2way(3), none(0)
  UPDATE #events 
    SET value=CASE LOWER(value) 
		WHEN 'totgt' THEN '1' 
		WHEN 'tosrc' THEN '2' 
		WHEN '2way' THEN '3' 
		ELSE '0' 
	      END
    WHERE root='link' and keyword='dir'

  DECLARE blnk_c CURSOR FOR SELECT DISTINCT evtid FROM #events ORDER BY evtid
  OPEN blnk_c
  FETCH NEXT FROM blnk_c INTO @evtid
  
  WHILE @@FETCH_STATUS <> -1 BEGIN
    SET @disp = ''
    SET @stat = 'OK'
    TRUNCATE TABLE #attrs
    TRUNCATE TABLE #contd_links
    INSERT #attrs SELECT root, keyword, value FROM #events WHERE evtid=@evtid
    INSERT #attrs SELECT 'CommonListener', 'instrId', @instrid

    -- gets all the valid combinations of links that are to be processed
    EXEC @ret = cl_parseLinks @name OUTPUT, @dir OUTPUT, @src_cid OUTPUT, @disp OUTPUT
    IF @ret >= 0 BEGIN
      INSERT #linkEventData SELECT @src_cid, src_id, dst_cid, @name, @dir, 0, 
				   dst_kn, dst_kv, 1, 0, NULL, 1 FROM #contd_links
      SET @cok = @cok + 1
    END
    ELSE SET @stat = 'ERROR'

    SET @cstr = @cstr + 1
    EXEC cl_logStatus @trid, @evtid, @disp, @stat
    FETCH NEXT FROM blnk_c INTO @evtid
  END
  DEALLOCATE blnk_c
  SET @disp = 'BULK Link Count: ' + @cstr + ', Processed OK: ' + @cok
  EXEC cl_logStatus @trid, 0, @disp, 'OK'

  -- Delete links that do not pertain to the instances created by this Instrumentation
  DELETE d FROM #linkEventData d LEFT OUTER JOIN CL_IDCache c 
    ON (c.cid=d.cid AND c.oid=d.iid AND c.instrID=@instrid AND c.deleted=0)
    WHERE c.cid IS NULL

  DECLARE blnk_c CURSOR FOR SELECT DISTINCT cid, linkname, src_direction, dest_cid, dest_keyname, dest_condition FROM #linkEventData
    WHERE linkname IN (SELECT link_type FROM link_type WHERE flag_containment=1 OR flag_parent_child=1)
  OPEN blnk_c
  FETCH NEXT FROM blnk_c INTO @src_cid, @name, @dir, @dst_cid, @kn, @kv

  WHILE @@FETCH_STATUS <> -1 BEGIN
    EXEC cl_getLinkDsts @dst_cid, @kn, @kv, 1, @src_cid

    IF EXISTS (SELECT * FROM #dst_oids) BEGIN
      INSERT #linkCandidates SELECT @name, @dir, @src_cid, a.iid, 0, @dst_cid, b.oid, 0
        FROM #linkEventData a CROSS JOIN #dst_oids b        
        WHERE cid=@src_cid AND linkname=@name AND src_direction=@dir AND 
              dest_cid=@dst_cid AND dest_keyname=@kn AND dest_condition=@kv

      -- Delete from #linkEventData, so that its not made link rule
      DELETE FROM #linkEventData 
        WHERE cid=@src_cid AND linkname=@name AND src_direction=@dir AND 
              dest_cid=@dst_cid AND dest_keyname=@kn AND dest_condition=@kv
    END

    FETCH NEXT FROM blnk_c INTO @src_cid, @name, @dir, @dst_cid, @kn, @kv
  END
  DEALLOCATE blnk_c

  -- contd link processing
  CREATE TABLE #existing_links (
    type NVARCHAR(255),
    src_cid ClassID,
    src_id ObjID,
    dst_cid ClassID,
    dst_id ObjID
  )
  -- get existing links for cids in this instrumentation
  INSERT #existing_links SELECT l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id
    FROM link l, CL_IDCache c WHERE c.cid=l.dst_cid AND c.oid=l.dst_id AND c.instrID=@instrid 
	 AND l.deleted=0 AND c.deleted=0 AND l.link_type IN 
	 (SELECT link_type FROM link_type WHERE flag_containment=1 OR flag_parent_child=1)
  
  -- get and delete links that exist in TBSM and are not in #linkCandidates
  DELETE e FROM #existing_links e, #linkCandidates c WHERE e.type=c.name AND 
    e.src_cid=c.dst_cid AND e.src_id=c.dst_id AND e.dst_cid=c.src_cid AND e.dst_id=c.src_id  

  DECLARE @src_id ObjID, @dst_id ObjID
  DECLARE del_c CURSOR FOR SELECT type, src_cid, src_id, dst_cid, dst_id FROM #existing_links 
  OPEN del_c
  FETCH NEXT FROM del_c INTO @name, @src_cid, @src_id, @dst_cid, @dst_id
 
  WHILE @@FETCH_STATUS <> -1 BEGIN 
    EXEC asisp_deletelink @type=@name, @src_cid=@src_cid, @src_id=@src_id, 
         @dst_cid=@dst_cid, @dst_id=@dst_id
    FETCH NEXT FROM del_c INTO @name, @src_cid, @src_id, @dst_cid, @dst_id
  END
  DEALLOCATE del_c


  -- process contd links 
  SET @disp = ''
  IF EXISTS (SELECT * FROM #linkCandidates)  
    EXEC cl_processContd @disp OUTPUT
  EXEC cl_logStatus @trid, 0, @disp, 'OK'
  
  -- Call link rule processing
--  IF EXISTS (SELECT * FROM #linkEventData)    
--    EXEC cl_processLinksInBulk
