SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[cl_processLinks]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[cl_processLinks]
GO




/* This procedure handles add/delete actions for links defined by the data in a temporary table. Handling of adds
   consists of defining a rule for the Link_Rules table, then processing the rule to create any new links. Handling
   of deletes requires identifying an existing rule, removing the links associated with the rule, then removing the
   rule.                                                                                                          

   The temporary table is defined as follows:                                                                    

     CREATE TABLE #linkEventData( cid                ClassID,
                                  iid                ObjID,
                                  dest_cid           ClassID NULL,
                                  linkname           NVARCHAR(128),
                                  src_direction      TINYINT,  
                                  dest_direction     TINYINT,
                                  dest_keyname       NVARCHAR(32),
                                  dest_condition     NVARCHAR(512),
                                  enabled            BIT,
                                  excl               BIT,
                                  priority           INT,
                                  action             INT )

    where all the columns but the last define the data for a link rule. The last column indicates whether an add (1)
    or delete(2) is required for a particular row.
 */

CREATE PROCEDURE cl_processLinks @disposition   NVARCHAR(255) OUTPUT
AS

RAISERROR( 'cl_processLinks', 0, 1 )

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN cl_processLinks                       
ELSE
  SAVE TRAN cl_processLinks                       


DECLARE @cid                ClassID 
DECLARE @iid                ObjID 
DECLARE @dest_cid           ClassID 
DECLARE @linkname           NVARCHAR(128) 
DECLARE @src_direction      TINYINT     
DECLARE @dest_direction     TINYINT     
DECLARE @dest_keyname       NVARCHAR(32) 
DECLARE @dest_condition     NVARCHAR(512) 
DECLARE @enabled            BIT 
DECLARE @excl               BIT 
DECLARE @priority           INT 
DECLARE @action             INT  

DECLARE @rule_id            INT

DECLARE @tempCount          INT
DECLARE @addCount           INT
DECLARE @deleteCount        INT
DECLARE @return             INT


SET @tempCount = 0
SET @addCount = 0
SET @deleteCount = 0
SET @return = 0

-- cursor through the input table                                 

DECLARE linkEventCursor CURSOR FOR SELECT * FROM #linkEventData  

OPEN linkEventCursor
FETCH NEXT FROM linkEventCursor INTO @cid,
                                     @iid,           
                                     @dest_cid,      
                                     @linkname,      
                                     @src_direction,     
                                     @dest_direction,     
                                     @dest_keyname,  
                                     @dest_condition,
                                     @enabled,       
                                     @excl,          
                                     @priority,      
                                     @action        

WHILE @@FETCH_STATUS = 0 AND @return !< 0 BEGIN

  SET @tempCount = @tempCount + 1

  EXEC @rule_id = cl_getLinkRule @linkname, 
                                 @src_direction,
                                 @cid,
                                 @iid,           
                                 @dest_direction,
                                 @dest_cid,      
                                 @dest_keyname,  
                                 @dest_condition 

  -- add or delete?
  IF ( @action = 1 ) AND ( @rule_id = 0 ) BEGIN

    -- its a new rule, so add and process                      
    EXEC @rule_id = cl_addLinkRule @linkname,       
                                   @src_direction,      
                                   @cid,            
                                   @iid,            
                                   @dest_direction,      
                                   @dest_cid,       
                                   @dest_keyname,   
                                   @dest_condition, 
                                   @enabled,
                                   @excl,   
                                   @priority

    -- process the link rule to produce a temporary table containing a list of resources identified by the rule.
    -- Then "call back" to a stored procedure to handle the resources as appropriate
    IF @rule_id <> 0 BEGIN
      -- process only non-containment links
      IF NOT EXISTS(SELECT * FROM link_type WHERE link_type=@linkname AND (flag_containment=1 OR flag_parent_child=1))
        EXEC @return = cl_processLinkRuleID @rule_id, 'cl_createLinks'
      IF @return !< 0 
        SET @addCount = @addCount + 1
    END
    ELSE 
      SET @return = -30000

  END

  ELSE
    IF ( @action = 2 ) AND ( @rule_id <> 0 ) BEGIN
      EXEC @return = cl_deleteLinksForRule @rule_id
      IF @return !< 0 BEGIN
        EXEC @return = cl_deleteLinkRule @rule_id
        IF @return !< 0
          SET @deleteCount = @deleteCount + 1
      END
    END

  FETCH NEXT FROM linkEventCursor INTO @cid,
                                       @iid,           
                                       @dest_cid,      
                                       @linkname,      
                                       @src_direction,     
                                       @dest_direction,     
                                       @dest_keyname,  
                                       @dest_condition,
                                       @enabled,       
                                       @excl,          
                                       @priority,      
                                       @action        

END
CLOSE linkEventCursor
DEALLOCATE linkEventCursor

IF @return < 0 BEGIN
  ROLLBACK TRAN cl_processLinks
  SET @disposition = 'transaction rolled back while handling temp data row: ' + CAST( @tempCount AS NVARCHAR(10) )
END
ELSE BEGIN
  SET @disposition = 'Total link event entries: ' + CAST( @tempCount AS NVARCHAR(10) )    + 
                     '  Rules added: '            + CAST( @addCount AS NVARCHAR(10) )     + 
                     '  Rules deleted: '          + CAST( @deleteCount AS NVARCHAR(10) )  

  -- commit transaction
  IF @tcount = 0                           
    COMMIT TRAN cl_processLinks
END

RETURN @return



GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

