PRINT 'Creating procedure cl_processNextTrans'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processNextTrans')
  DROP PROCEDURE cl_processNextTrans
GO

CREATE PROCEDURE cl_processNextTrans
@inTrID INT = 0
AS
  -- Main stored procedure that processes the next event
  RAISERROR('cl_processNextTrans: @inTrID=%d', 0, 1, @inTrID)

  CREATE TABLE #trids (
    trid INT UNIQUE, 
    trtype INT
  )
  CREATE TABLE #events (
    evtid INT,
    root NVARCHAR(512), 
    keyword NVARCHAR(64),
    value NVARCHAR(255)
  ) 
  CREATE TABLE #devents (
    evtid INT,
    action INT,
    root NVARCHAR(512), 
    keyword NVARCHAR(64),
    value NVARCHAR(255)
  ) 
  CREATE TABLE #attrs (
    root NVARCHAR(512), 
    keyword NVARCHAR(64),
    value NVARCHAR(255)    
  )
  CREATE TABLE #devtids ( 
    evtid INT, 
    id INT IDENTITY(1,1) 
  )
  CREATE TABLE #event_log (
    eid INT,
    status VARCHAR(16),
    disp NVARCHAR(1024)
  )
  CREATE TABLE #map_trans_evts (
    trid INT,
    evtid INT,
    eid INT IDENTITY(1,1)
  )


  DECLARE @trid INT, @trtype TINYINT

PROCESS_NEXT:

  -- For now, processing all transactions that have trState = 1
  IF @inTrID > 0
    INSERT #trids SELECT trId, trType FROM CL_Stage WHERE trState=1 AND trId=@inTrID
  ELSE
    INSERT #trids SELECT trId, trType FROM CL_Stage WHERE trState=1

/*
  -- Get first transaction (for instrId) if its trState=1
  INSERT #trids SELECT trId, trType FROM CL_Stage WHERE trState=1 AND trId IN 
    (SELECT MIN(trId) FROM CL_Stage GROUP BY registrationId)

  -- Get all transactions (for instrId) where trState=1 and its previous trState=5
  INSERT #trids SELECT s1.trId, s1.trType FROM CL_Stage s1
    JOIN CL_Stage s2 ON (s1.registrationId=s2.registrationId AND s1.trId > s2.trId)
    WHERE s1.trState=1 AND s2.trState=5 AND s2.trId IN 
    (SELECT MAX(s2.trId) FROM CL_Stage s1
      JOIN CL_Stage s2 ON (s1.registrationId=s2.registrationId AND s1.trId > s2.trId)
      WHERE s1.trState=1 GROUP BY s1.trId)
*/
 
  DECLARE tr_c CURSOR FOR SELECT trid, trtype FROM #trids ORDER BY trid
  OPEN tr_c
  FETCH NEXT FROM tr_c INTO @trid, @trtype

  WHILE @@FETCH_STATUS <> -1 BEGIN
    -- Mark trStage in CL_Stage as processing (4) & Log start-time of processing
    UPDATE CL_Stage SET trState=4 WHERE trId=@trid
    EXEC cl_logStatus @trid, 0, 'Started processing ...', 'OK'

    DECLARE @sql NVARCHAR(255), @disp VARCHAR(8000), @stat NVARCHAR(8), @ret INT
    DECLARE @evtid INT, @cid ClassID, @action TINYINT
    SET @stat = 'OK'
    SET @disp = ''

    -- get Instrumentation ID
    DECLARE @instrid NVARCHAR(255)
    SELECT @instrid=instrId FROM CL_Stage a, CL_Registration b
      WHERE a.trId=@trid AND a.registrationId=b.registrationId 

    -- BULK processing
    -- This involves add/delete/update of resources and EVENT processing
    -- Former is done here and latter in EVENT processing section
    IF @trtype = 1 BEGIN
      -- processing discovery data
      INSERT #events SELECT evDataId, root, keyword, value from CL_StageBulkData 
        WHERE trId=@trid AND root LIKE 'inst%'
      EXEC cl_processBulk @trid

      -- processing link data
      TRUNCATE TABLE #events
      INSERT #events SELECT evDataId, root, keyword, value from CL_StageBulkData 
        WHERE trId=@trid AND root LIKE 'link%'

      EXEC cl_processBulkLinks @trid, @instrid
    END
    ELSE IF @trtype = 2 BEGIN	-- DELTA processing    
      INSERT #devents SELECT evDataId, action, root, keyword, value FROM CL_StageDeltaData WHERE trId=@trid 
      
      -- Prep to process resources, links and events in that order
      TRUNCATE TABLE #devtids
      INSERT #devtids SELECT DISTINCT evtid FROM #devents WHERE action BETWEEN 1 AND 3 ORDER BY evtid
      INSERT #devtids SELECT DISTINCT evtid FROM #devents WHERE action BETWEEN 4 AND 5 ORDER BY evtid

      DECLARE delta_c CURSOR FOR SELECT evtid FROM #devtids ORDER BY id
      OPEN delta_c
      FETCH NEXT FROM delta_c INTO @evtid

      WHILE @@FETCH_STATUS <> -1 BEGIN
        SET @action = 3
        SELECT @action=action FROM #devents WHERE evtid=@evtid    
        SET @disp = ''
        SET @stat = 'OK'
        TRUNCATE TABLE #attrs

        INSERT #attrs SELECT root, keyword, value FROM #devents WHERE evtid=@evtid 
        INSERT #attrs SELECT 'CommonListener', 'instrId', @instrid  -- Used in CL_IDCache to delete resources		

        IF @action = 4 OR @action = 5
          EXEC @ret = cl_processLink @action, @disp OUTPUT
        ELSE BEGIN
          SELECT @cid=value FROM #attrs 
	    WHERE root='inst' AND (keyword='class' OR keyword='prdver')

          EXEC @ret = cl_validCID @cid
          IF @ret = 0 
 	    EXEC @ret = cl_processDelta @cid, @action, @disp OUTPUT
          ELSE BEGIN
            SET @stat = 'ERROR'
  	    SET @disp = @disp + ', Invalid ClassID'
          END
        END

        IF @ret < 0
          SET @stat = 'ERROR'
        EXEC cl_logStatus @trid, @evtid, @disp, @stat 
  
        FETCH NEXT FROM delta_c INTO @evtid
      END
      DEALLOCATE delta_c

      -- Processing events in Bulk
      INSERT #events SELECT evtid, root, keyword, value FROM #devents WHERE action=0
      EXEC cl_processBulkEvents
      INSERT CL_Status SELECT @trid, eid, status, disp, GETDATE() FROM #event_log
    END  

    -- EVENT processing
    -- This includes processing events from BULK and EVENT flows
    DECLARE @cstr NVARCHAR(128), @ntrid INT, @asbulk INT
    IF @trtype = 3 OR @trtype = 1 BEGIN	
      TRUNCATE TABLE #events
      TRUNCATE TABLE #map_trans_evts
      TRUNCATE TABLE #event_log

      IF @trtype = 1 
        INSERT #events SELECT evDataId, root, keyword, value from CL_StageBulkData 
          WHERE trId=@trid AND root LIKE 'event%'
      ELSE BEGIN
	-- Process all consecutive SendEvent transactions in Bulk
        SELECT @ntrid=MIN(trId) FROM CL_Stage WHERE trId>@trid AND (trType<>3 OR trState<>1)
        IF @ntrid IS NULL SELECT @ntrid=MAX(trId)+1 FROM CL_Stage WHERE trId>@trid
        SELECT * FROM CL_Stage WHERE trId>=@trid AND trId<@ntrid

	-- Processing more than one SendEvent transactions
        IF @@ROWCOUNT > 1 BEGIN
          SET @asbulk = 1	
          INSERT #map_trans_evts SELECT DISTINCT trId, evDataId FROM CL_StageEvData
            WHERE trId>=@trid AND trId<@ntrid ORDER BY trId, evDataId
          INSERT #events SELECT map.eid, sed.root, sed.keyword, sed.value FROM CL_StageEvData sed
            JOIN #map_trans_evts map ON (sed.trId=map.trid AND sed.evDataId=map.evtid)
          UPDATE CL_Stage SET trState=4 FROM CL_Stage cls JOIN #map_trans_evts map ON (map.trid=cls.trId)
        END
	ELSE
          INSERT #events SELECT evDataId, root, keyword, value from CL_StageEvData 
	    WHERE trId=@trid    
      END
  
      -- Processing events in Bulk
      EXEC cl_processBulkEvents
      IF @asbulk = 1 BEGIN
        INSERT CL_Status SELECT map.trid, map.evtid, status, disp, GETDATE() FROM #event_log el
	  JOIN #map_trans_evts map ON (map.eid=el.eid)

        UPDATE CL_Stage SET trState=5 FROM CL_Stage cls JOIN #map_trans_evts map ON (map.trid=cls.trId)
        DELETE #trids FROM #trids a JOIN #map_trans_evts map ON (a.trid=map.trid)
      END
      ELSE
        INSERT CL_Status SELECT @trid, eid, status, disp, GETDATE() FROM #event_log
    END

    -- Log event count for BULK transaction
    SELECT @cstr=COUNT(DISTINCT evtid) FROM #events
    SET @cstr = 'BULK Event Count: ' + @cstr
    IF @trtype = 1 EXEC cl_logStatus @trid, 0, @cstr, 'OK'

    -- Invalid processing type
    IF @trtype < 1 OR @trtype > 3
      EXEC cl_logStatus @trid, 0, 'Invalid Transaction type in CL_Stage', 'ERROR'

    -- Log end-time of processing & Update trState to be completed in CL_Stage 
    EXEC cl_logStatus @trid, 0, 'Processing completed ...', 'OK'
    UPDATE CL_Stage SET trState=5 WHERE trId=@trid

    -- Clean-up temporary tables
    TRUNCATE TABLE #events
    TRUNCATE TABLE #devents
    TRUNCATE TABLE #attrs
    TRUNCATE TABLE #devtids
    TRUNCATE TABLE #event_log
    TRUNCATE TABLE #map_trans_evts

    FETCH NEXT FROM tr_c INTO @trid, @trtype
  END  
  DEALLOCATE tr_c
  
  -- get the next set of transactions
  IF EXISTS (SELECT * FROM #trids) BEGIN
    TRUNCATE TABLE #trids
    GOTO PROCESS_NEXT
  END
/*
  ELSE BEGIN
    -- Log transactions that could not be processed
    -- Have to do something about these bad transactions
    INSERT CL_Status SELECT trId, 0, 'ERROR', 'Could not process transaction as its predecessor has a bad state', GETDATE() 
      FROM CL_Stage WHERE trState=1
  END
*/
