USE Object
GO

PRINT 'Creating procedure gemsp_addpvToLOBInst'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_addpvToLOBInst')
  DROP PROCEDURE gemsp_addpvToLOBInst
GO

CREATE PROCEDURE gemsp_addpvToLOBInst
@product NVARCHAR(128) = '',
@version NVARCHAR(128) = '',
@pproduct NVARCHAR(128) = '',
@pversion NVARCHAR(128) = '',
@pmatch NVARCHAR(255) = '',
@plobname NVARCHAR(255) = '',
@plobid ObjID = 0,
@name NVARCHAR(128) = '',
@memOptions NVARCHAR(64) = NULL,
@memPattern NVARCHAR(64) = NULL,
@memExpr NVARCHAR(64) = NULL,
@memTest NVARCHAR(64) = NULL,
@action NVARCHAR(16) = 'INSERT'
AS  
  /* The key in GEM_LOBInstLookup is the combination of (pcid, plobname, plobid)

     pcid is ClassID of parent....derived from pproduct and/or pversion
     plobname is the LOB name
     plobid is the LOB id of plobname. If 0 all the LOBs with that name are considered
     plobid is checked for validity....if found to be invalid its reset to 0
     pmatch defines the where-clause to get parent LOB
     
     New instance filtering conditions can be inserted/existing ones updated using the
     above key combination. 

     If any/all of the key combination needs to be updated, then the way to do it is through
     a DELETE and then INSERT

     A new key [NEWID(),@name] is created for each @name when @cid is LineOfBusiness 
     even if one exists with same [pcid, plobname, plobid, pmatch] combination
     Another separate key [NEWID()] is used for all other @cid(s)

   */  
  RAISERROR('gemsp_addpvToLOBInst: @product=%s, @version=%s, @pproduct=%s, @pversion=%s, @pmatch=%s, @plobname=%s, @plobid=%d, @name=%s, @memOptions=%s, @memPattern=%s, @memExpr=%s, @memTest=%s, @action=%s', 0, 1, 
	    @product, @version, @pproduct, @pversion, @pmatch, @plobname, @plobid, @name, @memOptions, @memPattern, @memExpr, @memTest, @action)
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_addpvToLOBInst
  ELSE
    SAVE TRAN gemsp_addpvToLOBInst

  DECLARE @cid ClassID, @pcid ClassID, @view TINYINT
  SELECT @cid=cid FROM GEMLookupCID WHERE prodname=@product AND version=@version
  SELECT @pcid=cid FROM GEMLookupCID WHERE prodname=@pproduct AND version=@pversion
  IF @cid IS NULL 
    SELECT @cid=cid FROM obj_class WHERE cname=@product OR cname=@version
  IF @pcid IS NULL 
    SELECT @pcid=cid FROM obj_class WHERE cname=@pproduct OR cname=@pversion

  SET @view = 0
  SET @action = UPPER(@action)

  IF @cid IS NOT NULL AND @pcid IS NOT NULL BEGIN  
    IF @plobid < 0 SET @plobid = 0
    IF @plobname IS NULL SET @plobname = ''
    IF @pmatch IS NULL SET @pmatch = ''

    IF @plobid > 0 BEGIN
      SELECT * FROM link WHERE dst_cid=@pcid AND src_cid='LOB' 
        AND src_id=@plobid AND link_type='LOBL' AND deleted=0

      IF @@ROWCOUNT <= 0 BEGIN
        RAISERROR(55034, 0, 1, @plobid, @pcid)
        SET @plobid = 0
      END
    END

    IF @plobid = 0 AND @plobname = '' AND @pmatch = '' BEGIN
      RAISERROR(55039, 0, 1)
      -- Finish up transaction
      IF @tcount = 0 
        COMMIT TRAN gemsp_addpvToLOBInst
      RETURN -1
    END

    IF @cid = 'LOB' AND (@name = '' OR @name IS NULL) BEGIN
      RAISERROR(55040, 0, 1)
      -- Finish up transaction
      IF @tcount = 0 
        COMMIT TRAN gemsp_addpvToLOBInst
      RETURN -1
    END

    DECLARE @lobkey NVARCHAR(255)
    IF @cid <> 'LOB'
      SELECT @lobkey=LOBKey FROM GEM_LOBInstLookup 
        WHERE cid=@pcid AND oid_where=@pmatch AND LOBName=@plobname AND 
	      lobid=@plobid AND LOBKey NOT LIKE '%,%'
    ELSE BEGIN
      SELECT @lobkey=LOBKey FROM GEM_LOBInstLookup 
        WHERE cid=@pcid AND oid_where=@pmatch AND LOBName=@plobname AND 
	      lobid=@plobid AND LOBKey LIKE '%,'+@name

      IF @lobkey IS NOT NULL AND @action <> 'DELETE' BEGIN
        RAISERROR(55042, 0, 1, @name, @pcid, @pmatch, @plobname, @plobid)
        -- Finish up transaction
        IF @tcount = 0 
          COMMIT TRAN gemsp_addpvToLOBInst
        RETURN
      END
    END
   
    IF @lobkey IS NOT NULL BEGIN
      IF @action = 'DELETE' BEGIN
        DELETE FROM GEM_InstFiltering WHERE cid=@cid AND LOBKey=@lobkey
        RAISERROR(55041, 0, 1, @cid, @pcid, @pmatch, @plobname, @plobid)
      END
      ELSE IF @action = 'INSERT' BEGIN
        INSERT GEM_InstFiltering VALUES (@cid, @lobkey, @memOptions, @memPattern, @memExpr, @memTest, 1, 0, 0)
        RAISERROR(55035, 0, 1, @memOptions, @memPattern, @memExpr, @memTest, @cid, @pcid, @pmatch, @plobname, @plobid)
      END
      ELSE IF @action = 'UPDATE' BEGIN
        UPDATE GEM_InstFiltering SET memOptions=@memOptions, memPattern=@memPattern, memExpr=@memExpr, memTest=@memTest 
	  WHERE cid=@cid AND LOBKey=@lobkey
        RAISERROR(55036, 0, 1, @memOptions, @memPattern, @memExpr, @memTest, @cid, @pcid, @pmatch, @plobname, @plobid)
      END   	 
      SET @view = 1
    END
    ELSE IF @action = 'INSERT' BEGIN
      SET @lobkey = NEWID()

      -- Special case when child is LineOfBusiness
      IF @cid = 'LOB' BEGIN
        IF @name <> '' AND @name IS NOT NULL 
	  SET @lobkey = @lobkey + ',' + @name

	SET @memOptions = NULL
	SET @memPattern = NULL
	SET @memExpr = NULL
	SET @memTest = NULL
	INSERT GEM_UpdateLOBView (cid, id, LOBKey, action) SELECT @cid, 0, @lobkey, 'CREATE'        
      END

      INSERT GEM_LOBInstLookup VALUES (@lobkey, @pcid, @pmatch, @plobname, @plobid)
      INSERT GEM_InstFiltering VALUES (@cid, @lobkey, @memOptions, @memPattern, @memExpr, @memTest, 1, 0, 0)      
      RAISERROR(55037, 0, 1, @cid, @name, @pcid, @pmatch, @plobname, @plobid, @memOptions, @memPattern, @memExpr, @memTest)
      SET @view = 1
    END
    ELSE
      RAISERROR(55030, 0, 1, @pcid, @pmatch, @plobname, @plobid, @cid, @name)

    -- UPDATE the LOB VIEW with all the existing instances
    IF @view = 1 AND @cid <> 'LOB'
      IF NOT EXISTS(SELECT * FROM GEM_UpdateLOBView WHERE cid=@cid AND id=0)
        INSERT GEM_UpdateLOBView (cid, id) SELECT @cid, 0
  END
  ELSE IF @cid IS NULL 
    RAISERROR(55029, 0, 1, @product, @version)    
  ELSE IF @pcid IS NULL 
    RAISERROR(55029, 0, 1, @pproduct, @pversion)    

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_addpvToLOBInst

