USE Object
GO

PRINT 'Creating procedure gemsp_putInLOB'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='gemsp_putInLOB')
  DROP PROCEDURE gemsp_putInLOB
GO

CREATE PROCEDURE gemsp_putInLOB
@cid ClassID,
@id ObjID,
@name NVARCHAR(255),
@lobkey NVARCHAR(255),
@action NVARCHAR(8),
@disposition NVARCHAR(128) OUTPUT
AS

  -- This procedure performs creation/deletion of LOBs AS per @action variable
  -- If CREATE, it creates LOB, if it does not already exist
  -- If DELETE, it deletes the LOB, if it exists
  RAISERROR('gemsp_putInLOB: @cid=%s, @id=%d, @name=%s, @lobkey=%s, @action=%s, @disposition OUTPUT', 0, 1, @cid, @id, @name, @lobkey, @action)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_putInLOB
  ELSE
    SAVE TRAN gemsp_putInLOB

  DECLARE @lobid ObjID, @plobid ObjID, @tlobkey NVARCHAR(255), @plobkey NVARCHAR(255)
  DECLARE @newid ObjID, @lobname NVARCHAR(255), @exists TINYINT
  SET @tlobkey = @lobkey

  SELECT @lobid=LOBID, @plobid=parentLOBID, @plobkey=parentLOBKey, @lobname=LOBName 
    FROM GEM_LOBLookup WHERE LOBKey=@tlobkey
  IF @@ROWCOUNT <= 0 BEGIN	-- goto the other LOB lookup table
    EXEC gemsp_putInLOBInst @cid, @id, @name, @lobkey, @action, @disposition OUTPUT

    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_putInLOB
    RETURN
  END

  IF @action = 'DELETE' BEGIN
    IF @lobid > 0 AND @plobid > 0
      EXEC gemsp_action_inLOB 'DELETE', @cid, @id, @lobid, @disposition OUTPUT

    -- Finish up transaction
    IF @tcount = 0 
      COMMIT TRAN gemsp_putInLOB
    RETURN
  END
  
  WHILE @lobid = 0 OR @plobid = 0 BEGIN
    -- Update plobid if it is already created
    IF @plobid = 0 BEGIN
      DECLARE @created ObjID
      SELECT @created=LOBID FROM GEM_LOBLookup WHERE LOBKey=@plobkey
      IF @created > 0 BEGIN
        UPDATE GEM_LOBLookup SET parentLOBID=@created WHERE LOBKey=@tlobkey
        SELECT @plobid=parentLOBID FROM GEM_LOBLookup WHERE LOBKey=@tlobkey
      END
    END  

    -- Create LOB when parent is known (already created)
    IF @lobid = 0 AND @plobid > 0 BEGIN
      IF @plobkey = 'LOBC' BEGIN
        EXEC @newid = gemsp_createLOB @lobname, @tlobkey, @plobid, 'LOBC' 
        RAISERROR(55002, 0, 1, @cid, @id, @lobname, @tlobkey, @plobid)
      END
      ELSE BEGIN
        EXEC @newid = gemsp_createLOB @lobname, @tlobkey, @plobid
        RAISERROR(55003, 0, 1, @cid, @id, @lobname, @tlobkey, @plobid)
      END

      SET @tlobkey = @lobkey
      SELECT @lobid=LOBID, @plobid=parentLOBID, @plobkey=parentLOBKey, @lobname=LOBName 
        FROM GEM_LOBLookup WHERE LOBKey=@lobkey
    END        
    ELSE BEGIN	-- increment to parent
      SELECT @tlobkey=parentLOBKey FROM GEM_LOBLookup WHERE LOBKey=@tlobkey
      IF NOT EXISTS (SELECT * FROM GEM_LOBLookup WHERE LOBKey=@tlobkey) BEGIN
        RAISERROR(55004, 0, 1, @tlobkey, @cid, @id) 
        BREAK
      END
      ELSE
        SELECT @lobid=LOBID, @plobid=parentLOBID, @plobkey=parentLOBKey, @lobname=LOBName 
          FROM GEM_LOBLookup WHERE LOBKey=@tlobkey
    END
  END  -- end of while

  SELECT @lobid=LOBID, @plobid=parentLOBID FROM GEM_LOBLookup WHERE LOBKey=@lobkey

  -- Create & map a resource to LOB
  IF @lobid > 0 AND @plobid > 0 BEGIN
    DECLARE @child_id ObjID, @sql2 NVARCHAR(510), @tblname ObjName
    EXEC @exists = gemsp_action_inLOB 'EXISTS', @cid, @id, @lobid, @disposition OUTPUT
    
    IF @exists = 0 BEGIN
      EXEC @child_id = asisp_createLOB @name, 'LOB', @lobid
      RAISERROR(55005, 0, 1, @cid, @id, @name, @lobid)
 
      -- Attach the LOB to instance
      IF @child_id > 0 BEGIN
        SELECT @tblname = 'LOB_' + RTRIM(@cid) + '_LOBL'
        SELECT @sql2 = 'INSERT ' + @tblname + '(src_id, dst_id) SELECT ' + CONVERT(NCHAR(9), @child_id) + ', ' + CONVERT(NCHAR(9),@id)
        EXEC(@sql2)
        
        IF @disposition IS NULL
          SELECT @disposition = 'LOB : addedTo ' + CAST(@lobid AS NVARCHAR) + ' '
        ELSE
          SELECT @disposition = @disposition + ': addedTo ' + CAST(@lobid AS NVARCHAR) + ' '
      END
      ELSE
        RAISERROR(55006, 0, 1, 'CREATE')        
    END
  END

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN gemsp_putInLOB
